#!/usr/bin/env python3
"""Test the files module."""

# Standard imports
import unittest
import os
import sys


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
if EXEC_DIR.endswith('/pattoo-shared/tests/test_pattoo_shared') is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''\
This script is not installed in the "pattoo-shared/tests/test_pattoo_shared" \
directory. Please fix.''')
    sys.exit(2)

# Pattoo imports
from pattoo_shared import configuration
from pattoo_shared.variables import PollingTarget
from tests.libraries.configuration import UnittestConfig


class TestConfig(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    config = configuration.Config()

    def test___init__(self):
        """Testing function __init__."""
        pass

    def test_api_listen_address(self):
        """Testing function api_listen_address."""
        # Initialize key values
        expected = '0.0.0.0'

        # Test
        result = self.config.api_listen_address()
        self.assertEqual(result, expected)

    def test_polling_interval(self):
        """Testing function polling_interval."""
        # Initialize key values
        expected = 20

        # Test
        result = self.config.polling_interval()
        self.assertEqual(result, expected)

    def test_api_ip_address(self):
        """Testing function api_ip_address."""
        # Initialize key values
        expected = '127.0.0.1'

        # Test
        result = self.config.api_ip_address()
        self.assertEqual(result, expected)

    def test_api_ip_bind_port(self):
        """Testing function api_ip_bind_port."""
        # Initialize key values
        expected = 6060

        # Test
        result = self.config.api_ip_bind_port()
        self.assertEqual(result, expected)

    def test_api_uses_https(self):
        """Testing function api_uses_https."""
        # Initialize key values
        expected = False

        # Test
        result = self.config.api_uses_https()
        self.assertEqual(result, expected)

    def test_api_uri(self):
        """Testing function api_uri."""
        # Initialize key values
        expected = '/pattoo/api/v1/agent/receive'

        # Test
        result = self.config.api_uri()
        self.assertEqual(result, expected)

    def test_api_server_url(self):
        """Testing function api_server_url."""
        # Initialize key values
        expected = 'http://127.0.0.1:6060/pattoo/api/v1/agent/receive/123'
        agent_id = 123

        # Test
        result = self.config.api_server_url(agent_id)
        self.assertEqual(result, expected)

    def test_daemon_directory(self):
        """Testing function daemon_directory."""
        # Nothing should happen. Directory exists in testing.
        _ = self.config.daemon_directory()

    def test_log_directory(self):
        """Testing function log_directory."""
        # Nothing should happen. Directory exists in testing.
        _ = self.config.log_directory()

    def test_log_file(self):
        """Testing function log_file."""
        # Initialize key values
        expected = '{1}{0}pattoo.log'.format(
            os.sep, self.config.log_directory())

        # Test
        result = self.config.log_file()
        self.assertEqual(result, expected)

    def test_log_file_api(self):
        """Testing function log_file_api."""
        # Initialize key values
        expected = '{1}{0}pattoo-api.log'.format(
            os.sep, self.config.log_directory())

        # Test
        result = self.config.log_file_api()
        self.assertEqual(result, expected)

    def test_log_level(self):
        """Testing function log_level."""
        # Initialize key values
        expected = 'debug'

        # Test
        result = self.config.log_level()
        self.assertEqual(result, expected)

    def test_log_file_daemon(self):
        """Testing function log_file_daemon."""
        # Initialize key values
        expected = '{1}{0}pattoo-daemon.log'.format(
            os.sep, self.config.log_directory())

        # Test
        result = self.config.log_file_daemon()
        self.assertEqual(result, expected)

    def test_cache_directory(self):
        """Testing function cache_directory."""
        # Nothing should happen. Directory exists in testing.
        _ = self.config.cache_directory()

    def test_agent_cache_directory(self):
        """Testing function agent_cache_directory."""
        # Initialize key values
        agent_id = 123
        expected = '{1}{0}{2}'.format(
            os.sep, self.config.cache_directory(), agent_id)

        # Test
        result = self.config.agent_cache_directory(agent_id)
        self.assertEqual(result, expected)

    def test__polling_targets(self):
        """Testing function _polling_targets."""
        # Initialize key values
        oids = ['.1.3.6.1.2.1.2.2.1.10', '.1.3.6.1.2.1.2.2.1.16']
        data = [
            {'address': '.1.3.6.1.2.1.2.2.1.10',
             'multiplier': 8},
            {'address': '.1.3.6.1.2.1.2.2.1.16',
             'multiplier': 8}]

        # Test with good data
        result = self.config._polling_targets(data)
        self.assertTrue(isinstance(result, list))
        self.assertTrue(bool(result))
        for index, value in enumerate(result):
            self.assertTrue(isinstance(value, PollingTarget))
            self.assertEqual(value.address, oids[index])
            self.assertEqual(value.multiplier, 8)


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test_search(self):
        """Testing function search."""
        # Initialize key variables
        data = {
            1: {
                11: '11',
                12: '12'
            },
            2: {
                21: '21',
                22: '22'
            },
            3: {
                31: '31',
                32: '32'
            },
            4: {
                41: '41',
                42: '42'
            }
        }

        # Test OK value
        expected = '11'
        result = configuration.search(1, 11, data)
        self.assertEqual(result, expected)

        # Test all values
        for key, key_dict in data.items():
            for sub_key, expected in key_dict.items():
                result = configuration.search(key, sub_key, data)
                self.assertEqual(result, expected)

        # Test bad values
        with self.assertRaises(SystemExit):
            _ = configuration.search('1111111', 11, data)

        # Test bad values
        _ = configuration.search('1111111', 11, data, die=False)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Test the converter module."""

# Standard imports
import unittest
import os
import sys
import time
import json

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
if EXEC_DIR.endswith('/pattoo-shared/tests/test_pattoo_shared') is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''\
This script is not installed in the "pattoo-shared/tests/test_pattoo_shared" \
directory. Please fix.''')
    sys.exit(2)

# Pattoo imports
from pattoo_shared import converter
from pattoo_shared import data as lib_data
from pattoo_shared.variables import (
    DataPointMeta, DataPoint, DeviceDataPoints, DeviceGateway, AgentPolledData)
from pattoo_shared.configuration import Config
from pattoo_shared.constants import (
    DATA_FLOAT, DATA_INT, DATA_COUNT64, DATA_COUNT, DATA_STRING, DATA_NONE,
    PattooDBrecord)
from tests.libraries.configuration import UnittestConfig


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test_cache_to_keypairs(self):
        """Testing method / function cache_to_keypairs."""
        self.maxDiff = None
        cache = [
            "1234",
            [{"metadata": [
                {"agent_hostname": "palisadoes"},
                {"agent_id": "1234"},
                {"agent_program": "program_1"},
                {"device": "device_1"},
                {"gateway": "palisadoes"},
                {"polling_interval": "10"}],
              "data_label": 30386,
              "data_type": 99,
              "data_index": "index_1",
              "data_value": 523.0,
              "data_timestamp": 1574011824387,
              "checksum": '123'}
             ]
            ]

        # Test
        # cache = json.loads(cache_data)
        results = converter.cache_to_keypairs(cache[0], cache[1])
        self.assertTrue(isinstance(results, list))
        for _, result in enumerate(results):
            self.assertEqual(result.checksum, '''\
5c531afb2e4106cc78f99ff895ef99d2fa587f7272a91f82244f06caddf89176c299afc0ef4a44\
7d6c5042dff4690e271a3d09aabf97465faba20591f818ab27''')
            self.assertEqual(result.data_timestamp, 1574011824387)
            self.assertEqual(result.data_value, 523.0)
            self.assertEqual(result.data_index, 'index_1')
            self.assertEqual(result.data_type, 99)
            self.assertEqual(result.data_label, 30386)
            _metadata = cache[1][0]['metadata']
            self.assertEqual(
                result.metadata,
                [(_k_, _v_) for _dict in _metadata for _k_, _v_ in sorted(
                    _dict.items())])

    def test_agentdata_to_datapoints(self):
        """Testing method / function agentdata_to_datapoints."""
        # Setup AgentPolledData
        agent_id = 'koala_bear'
        agent_program = 'panda_bear'
        agent_hostname = 'localhost'
        polling_interval = 30
        apd = AgentPolledData(
            agent_id, agent_program, agent_hostname, polling_interval)

        # Initialize DeviceGateway
        gateway = 'grizzly_bear'
        dgw = DeviceGateway(gateway)

        # Initialize DeviceDataPoints
        device = 'teddy_bear'
        ddv = DeviceDataPoints(device)

        # Setup DataPoint
        value = 457
        data_label = 'gummy_bear'
        data_index = 999
        data_type = DATA_INT
        variable = DataPoint(
            value, data_label=data_label, data_index=data_index,
            data_type=data_type)

        # Add data to DeviceDataPoints
        ddv.add(variable)

        # Add data to DeviceGateway
        dgw.add(ddv)

        # Test DeviceGateway to AgentPolledData
        apd.add(dgw)

        # Test contents
        expected_metadata = {
            'agent_id': agent_id,
            'agent_program': agent_program,
            'agent_hostname': agent_hostname,
            'polling_interval': polling_interval,
            'gateway': gateway,
            'device': device
        }
        result = converter.agentdata_to_datapoints(apd)

        self.assertEqual(len(result), 1)
        item = result[0]
        self.assertTrue(isinstance(item, DataPoint))
        self.assertEqual(item.data_value, value)
        self.assertEqual(item.data_type, DATA_INT)
        self.assertEqual(item.data_index, data_index)
        self.assertEqual(item.data_label, data_label)
        self.assertEqual(
            item.checksum,
            '''\
adaa977bbc2f3b0cae66f4c3021963a34ef16077e3ad54d2ae3736b3842c85b0''')
        self.assertTrue(isinstance(item.metadata, dict))
        self.assertEqual(len(item.metadata), len(expected_metadata))
        for key, value in item.metadata.items():
            self.assertTrue(isinstance(value, str))
            self.assertTrue(isinstance(key, str))
            self.assertEqual(value, str(expected_metadata[key]))

    def test_datapoints_to_dicts(self):
        """Testing method / function datapoints_to_dicts."""
        # Initialize key variables
        datapoints = []
        now = time.time()

        # Create DataPoints
        for value in range(0, 5):
            metadata = []
            for meta in range(0, 22, 7):
                metadata.append(DataPointMeta(int(meta), str(meta * 2)))
            datapoint = DataPoint(
                value,
                data_index=(value * 10),
                data_type=('type_{}'.format(value)),
                data_label=('label_{}'.format(value)),
            )
            for meta in metadata:
                datapoint.add(meta)
            datapoints.append(datapoint)

        # Start testing
        result = converter.datapoints_to_dicts(datapoints)
        self.assertTrue(isinstance(result, list))
        for value, item in enumerate(result):
            self.assertEqual(item['data_index'], value * 10)
            self.assertEqual(item['data_type'], 'type_{}'.format(value))
            self.assertEqual(item['data_label'], 'label_{}'.format(value))
            self.assertTrue(item['data_timestamp'] >= now)
            self.assertTrue(item['data_timestamp'] <= now + 1000)
            self.assertTrue(isinstance(item['checksum'], str))
            self.assertTrue(isinstance(item['metadata'], list))
            self.assertTrue(len(item['metadata'], 4))
            for meta, metadata in enumerate(item['metadata']):
                self.assertTrue(metadata, dict)
                for m_key, m_value in metadata.items():
                    self.assertEqual(2 * int(m_key), int(m_value))
                    self.assertEqual(int(m_key) % 7, 0)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Pattoo times library."""

import time

# Pattoo libraries
from pattoo_shared import data
from pattoo_shared import log


def validate_timestamp(timestamp, polling_interval):
    """Validate timestamp to be a multiple of 'polling_interval' seconds.

    Args:
        timestamp: epoch timestamp in seconds
        polling_interval: Polling interval for data

    Returns:
        valid: True if valid

    """
    # Initialize key variables
    valids = []
    valid = False

    # Evaluate validity of the parameters
    valids.append(bool(polling_interval))
    valids.append(data.is_numeric(timestamp))
    valids.append(data.is_numeric(polling_interval))
    valids.append(isinstance(polling_interval, (int, float)))

    # Process data
    if False not in valids:
        test = (int(timestamp) // polling_interval) * polling_interval
        if test == timestamp:
            valid = True

    # Return
    return valid


def normalized_timestamp(_polling_interval, timestamp=None):
    """Normalize timestamp to a multiple of 'polling_interval' seconds.

    Args:
        timestamp: epoch timestamp in seconds
        _polling_interval: Polling interval for data

    Returns:
        value: Normalized value

    """
    # Initialize key variables
    if isinstance(_polling_interval, int) is False or (
            _polling_interval is True or _polling_interval is False):
        log_message = (
            'Invalid non-integer value for {}'.format(_polling_interval))
        log.log2die(1029, log_message)
    else:
        polling_interval = abs(_polling_interval)

    # Don't allow 0 values for polling_interval
    if bool(polling_interval) is False:
        polling_interval = 1

    # Process data
    if (timestamp is None) or (data.is_numeric(timestamp) is False):
        value = (int(time.time()) // polling_interval) * polling_interval
    else:
        value = (int(timestamp) // polling_interval) * polling_interval

    # Return
    return (value, polling_interval)



#!/usr/bin/env python3
"""Pattoo HTTP data classes."""

# Standard libraries
import os
import sys
import json
import urllib
from time import time

# pip3 libraries
import requests

# Pattoo libraries
from pattoo_shared import log
from pattoo_shared.configuration import Config
from pattoo_shared import converter


class Post(object):
    """Class to prepare data for posting to remote pattoo server."""

    def __init__(self, identifier, identifier_data=None):
        """Initialize the class.

        Args:
            identifier: Unique identifer for the source of the data.
            identifier_data: Data from the data source to post

        Returns:
            None

        """
        # Initialize key variables
        config = Config()

        # Test validity
        if isinstance(identifier, str) is False:
            log_message = ('''\
Data identifier isn\'t a string. Identifier: {}'''.format(identifier))
            log.log2die(1018, log_message)

        # Get posting URL
        self._identifier = identifier
        self._url = config.api_server_url(identifier)
        self._cache_dir = config.agent_cache_directory(identifier)
        self._identifier_data = identifier_data

    def post(self, save=True, data=None):
        """Post data to central server.

        Args:
            save: When True, save data to cache directory if postinf fails
            data: Data to post. If None, then uses self._post_data (
                Used for testing and cache purging)

        Returns:
            success: True: if successful

        """
        # Initialize key variables
        success = False
        response = False
        valid_data = False

        # Create data to post
        if bool(data) is True:
            valid_data = data
        else:
            valid_data = converter.datapoints_to_dicts(self._identifier_data)

        # Fail if nothing to post
        if isinstance(valid_data, list) is False or bool(valid_data) is False:
            return success

        # Post data save to cache if this fails
        try:
            result = requests.post(self._url, json=valid_data)
            response = True
        except:
            if save is True:
                # Save data to cache
                self._save(valid_data)
            else:
                # Proceed normally if there is a failure.
                # This will be logged later
                pass

        # Define success
        if response is True:
            if result.status_code == 200:
                success = True
            else:
                log_message = (
                    'HTTP {} error for identifier "{}" posted to server {}'
                    ''.format(result.status_code, self._identifier, self._url))
                log.log2debug(1017, log_message)
                # Save data to cache, remote webserver isn't working properly
                self._save(valid_data)

        # Log message
        if success is True:
            log_message = (
                'Data for identifier "{}" posted to server {}'
                ''.format(self._identifier, self._url))
            log.log2debug(1027, log_message)
        else:
            log_message = (
                'Data for identifier "{}" failed to post to server {}'
                ''.format(self._identifier, self._url))
            log.log2warning(1028, log_message)

        # Return
        return success

    def purge(self):
        """Purge data from cache by posting to central server.

        Args:
            None

        Returns:
            success: "True: if successful

        """
        # Initialize key variables
        identifier = self._identifier

        # Add files in cache directory to list only if they match the
        # cache suffix
        all_filenames = [filename for filename in os.listdir(
            self._cache_dir) if os.path.isfile(
                os.path.join(self._cache_dir, filename))]
        filenames = [
            filename for filename in all_filenames if filename.endswith(
                '.json')]

        # Read cache file
        for filename in filenames:
            # Only post files for our own UID value
            if identifier not in filename:
                continue

            # Get the full filepath for the cache file and post
            filepath = os.path.join(self._cache_dir, filename)
            with open(filepath, 'r') as f_handle:
                try:
                    data = json.load(f_handle)
                except:
                    # Log removal
                    log_message = (
                        'Error reading previously cached agent data file {} '
                        'for identifier {}. May be corrupted.'
                        ''.format(filepath, self._identifier))
                    log.log2die(1064, log_message)

            # Post file
            success = self.post(save=False, data=data)

            # Delete file if successful
            if success is True:
                os.remove(filepath)

                # Log removal
                log_message = (
                    'Purging cache file {} after successfully '
                    'contacting server {}'
                    ''.format(filepath, self._url))
                log.log2info(1007, log_message)

    def _save(self, data):
        """Save data to cache file.

        Args:
            cache_dir: Cache directory
            identifier: Identifier
            data: Dict to save

        Returns:
            success: True: if successful

        """
        # Initialize key variables
        cache_dir = self._cache_dir
        identifier = self._identifier
        timestamp = int(time() * 1000)

        # Create a unique very long filename to reduce risk of
        filename = '{}/{}_{}.json'.format(cache_dir, timestamp, identifier)

        # Save data
        with open(filename, 'w') as f_handle:
            json.dump(data, f_handle)


class PostAgent(Post):
    """Class to prepare data for posting to remote pattoo server."""

    def __init__(self, agentdata):
        """Initialize the class.

        Args:
            identifier: Unique identifer for the source of the data.
            identifier_data: Data from the data source to post

        Returns:
            None

        """
        # Get extracted data
        data = converter.agentdata_to_datapoints(agentdata)

        # Initialize key variables
        Post.__init__(
            self, agentdata.agent_id, identifier_data=data)


class PassiveAgent(object):
    """Class to handle data from passive Pattoo Agents."""

    def __init__(self, url):
        """Initialize the class.

        Args:
            url: URL to get

        Returns:
            None

        """
        # Initialize key variables
        self._url = url

    def relay(self):
        """Forward data polled from remote pattoo passive agent.

        Args:
            None

        Returns:
            None

        """
        # Get data
        data_dict = self.get()

        # Post data
        if bool(data_dict) is True:
            # Post to remote server
            identifier = converter.convert(data_dict)
            server = Post(identifier)
            success = server.post()

            # Purge cache if success is True
            if success is True:
                server.purge()

    def get(self):
        """Get JSON from remote URL.

        Args:
            None

        Returns:
            result: dict of JSON retrieved.

        """
        # Initialize key variables
        result = {}
        url = self._url

        # Get URL
        try:
            with urllib.request.urlopen(url) as u_handle:
                try:
                    result = json.loads(u_handle.read().decode())
                except:
                    error = sys.exc_info()[:2]
                    log_message = (
                        'Error reading JSON from URL {}: ({} {})'
                        ''.format(url, error[0], error[1]))
                    log.log2info(1008, log_message)
        except:
            # Most likely no connectivity or the TCP port is unavailable
            error = sys.exc_info()[:2]
            log_message = (
                'Error contacting URL {}: ({} {})'
                ''.format(url, error[0], error[1]))
            log.log2info(1186, log_message)

        # Return
        return result



#!/usr/bin/env python3
"""Test the files module."""

# Standard imports
import unittest
import os
import sys

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
if EXEC_DIR.endswith('/pattoo-shared/tests/test_pattoo_shared') is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''\
This script is not installed in the "pattoo-shared/tests/test_pattoo_shared" \
directory. Please fix.''')
    sys.exit(2)

# Pattoo imports
from tests.libraries.configuration import UnittestConfig

from pattoo_shared.constants import DATA_FLOAT
from pattoo_shared.constants import DATA_INT
from pattoo_shared.constants import DATA_COUNT64
from pattoo_shared.constants import DATA_COUNT
from pattoo_shared.constants import DATA_STRING
from pattoo_shared.constants import DATA_NONE

from pattoo_shared.constants import PATTOO_API_SITE_PREFIX
from pattoo_shared.constants import PATTOO_API_AGENT_PREFIX
from pattoo_shared.constants import PATTOO_API_AGENT_EXECUTABLE
from pattoo_shared.constants import PATTOO_API_AGENT_PROXY

from pattoo_shared.constants import PATTOO_AGENT_OS_SPOKED_API_PREFIX
from pattoo_shared.constants import PATTOO_AGENT_OS_SPOKED
from pattoo_shared.constants import PATTOO_AGENT_OS_SPOKED_PROXY
from pattoo_shared.constants import PATTOO_AGENT_OS_AUTONOMOUSD
from pattoo_shared.constants import PATTOO_AGENT_OS_HUBD
from pattoo_shared.constants import PATTOO_AGENT_SNMPD
from pattoo_shared.constants import PATTOO_AGENT_MODBUSTCPD
from pattoo_shared.constants import PATTOO_AGENT_BACNETIPD


class TestConstants(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test_constants(self):
        """Testing constants."""
        # Test data type constants
        self.assertEqual(DATA_FLOAT, 101)
        self.assertEqual(DATA_INT, 99)
        self.assertEqual(DATA_COUNT64, 64)
        self.assertEqual(DATA_COUNT, 32)
        self.assertEqual(DATA_STRING, 2)
        self.assertEqual(DATA_NONE, None)

        # Test pattoo API constants
        self.assertEqual(
            PATTOO_API_SITE_PREFIX, '/pattoo/api/v1')
        self.assertEqual(
            PATTOO_API_AGENT_PREFIX, '{}/agent'.format(PATTOO_API_SITE_PREFIX))
        self.assertEqual(
            PATTOO_API_AGENT_EXECUTABLE, 'pattoo-api-agentd')
        self.assertEqual(
            PATTOO_API_AGENT_PROXY,
            '{}-gunicorn'.format(PATTOO_API_AGENT_EXECUTABLE))

        # Test agent constants
        self.assertEqual(
            PATTOO_AGENT_OS_SPOKED_API_PREFIX, '/pattoo-agent-os')
        self.assertEqual(
            PATTOO_AGENT_OS_SPOKED, 'pattoo-agent-os-spoked')
        self.assertEqual(
            PATTOO_AGENT_OS_SPOKED_PROXY,
            '{}-gunicorn'.format(PATTOO_AGENT_OS_SPOKED))
        self.assertEqual(
            PATTOO_AGENT_OS_AUTONOMOUSD, 'pattoo-agent-os-autonomousd')
        self.assertEqual(
            PATTOO_AGENT_OS_HUBD, 'pattoo-agent-os-hubd')
        self.assertEqual(
            PATTOO_AGENT_SNMPD, 'pattoo-agent-snmpd')
        self.assertEqual(
            PATTOO_AGENT_MODBUSTCPD, 'pattoo-agent-modbustcpd')
        self.assertEqual(
            PATTOO_AGENT_BACNETIPD, 'pattoo-agent-bacnetipd')


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Pattoo Data Converter."""

# Pattoo libraries
from .variables import (
    DataPointMeta, DataPoint, AgentPolledData)
from .constants import (
    DATA_FLOAT, DATA_INT, DATA_COUNT64, DATA_COUNT, DATA_STRING, DATA_NONE,
    MAX_KEYPAIR_LENGTH, PattooDBrecord)
from pattoo_shared import data


def cache_to_keypairs(data_source, items):
    """Convert agent cache data to AgentPolledData object.

    Args:
        items: Cache data
        data_source: Source of the cache data

    Returns:
        result: Validated cache data. [] if invalid.

    """
    # Initialize key variables
    result = []

    # Verify we are getting a list
    if isinstance(items, list) is False:
        return []

    # Verify contents of lists
    for item in items:
        # Initialize data
        valids = []
        keypairs = []

        # Verify dicts are in the list
        valids.append(isinstance(item, dict))

        # Get all the key-pairs for the item
        keypairs = []
        for key, value in sorted(item.items()):
            # All the right keys
            keys = ['checksum', 'metadata', 'data_type', 'data_label',
                    'data_index', 'data_value', 'data_timestamp']
            valids.append(key in keys)
            valids.append(len(item) == len(keys))
            valids.append(isinstance(key, str))

            # Work on metadata
            if key == 'metadata':
                # Must be a dict
                if isinstance(value, list) is False:
                    valids.append(False)
                    continue

                # Add metadata keypairs as a list of tuples
                for keypair in value:
                    for m_key, m_value in keypair.items():
                        if isinstance(m_key, str) is False:
                            continue
                        if isinstance(m_value, str) is False:
                            continue
                        keypairs.append(
                            (str(m_key)[:MAX_KEYPAIR_LENGTH], str(
                                m_value)[:MAX_KEYPAIR_LENGTH])
                        )

            # Work on the data_type
            if key == 'data_type':
                valids.append(value in [
                    DATA_FLOAT, DATA_INT, DATA_COUNT64, DATA_COUNT,
                    DATA_STRING, DATA_NONE])

        # Append to result
        if False not in valids:
            # Add the datasource to the original checksum for better uniqueness
            checksum = data.hashstring(
                '{}{}'.format(data_source, item['checksum']), sha=512)
            pattoo_db_variable = PattooDBrecord(
                checksum=checksum,
                data_index=item['data_index'],
                data_label=item['data_label'],
                data_source=data_source,
                data_timestamp=item['data_timestamp'],
                data_type=item['data_type'],
                data_value=item['data_value'],
                metadata=keypairs
            )
            result.append(pattoo_db_variable)

    # Return
    return result


def agentdata_to_datapoints(agentdata):
    """Ingest data.

    Args:
        agentdata: AgentPolledData object

    Returns:
        rows: List of DataPoint objects

    """
    # Initialize key variables
    rows = []

    # Only process valid data
    if isinstance(agentdata, AgentPolledData) is True:
        # Return if invalid data
        if bool(agentdata.valid) is False:
            return []

        # Cycle through the data
        for gwd in agentdata.data:
            # Ignore bad data
            if gwd.valid is False:
                continue

            for ddv in gwd.data:
                # Ignore bad data
                if ddv.valid is False:
                    continue

                # Get data
                for _dv in ddv.data:
                    # Assign values to DataPoints
                    metadata = {
                        'agent_id': agentdata.agent_id,
                        'agent_program': agentdata.agent_program,
                        'agent_hostname': agentdata.agent_hostname,
                        'polling_interval': agentdata.polling_interval,
                        'gateway': gwd.device,
                        'device': ddv.device,
                        'device_type': ddv.device_type
                    }
                    for key, value in metadata.items():
                        _dv.add(DataPointMeta(key, value))
                    rows.append(_dv)

    # Return
    return rows


def datapoints_to_dicts(items):
    """Ingest data.

    Args:
        items: List of datapoints to convert

    Returns:
        result: List of datapoints converted to a list of dicts

    """
    # Initialize key variables
    datapoints = []
    result = []

    # Verify input data
    if isinstance(items, list) is False:
        items = [items]
    for item in items:
        if isinstance(item, DataPoint):
            datapoints.append(item)

    # Convert to a list of dicts
    for datapoint in datapoints:
        # Only convert valid data
        if datapoint.valid is True:
            data_dict = {
                'metadata': [
                    {str(key): str(value)} for key, value in sorted(
                        datapoint.metadata.items())],
                'data_label': datapoint.data_label,
                'data_type': datapoint.data_type,
                'data_index': datapoint.data_index,
                'data_value': datapoint.data_value,
                'data_timestamp': datapoint.data_timestamp,
                'checksum': datapoint.checksum
            }
            result.append(data_dict)

    return result



"""Module that defines constants shared between pattoo and its agents.

The aim is to have a single location for constants that may be used across
agents to prevent the risk of duplication.

"""
import collections

###############################################################################
# Universal constants for all agents
###############################################################################

DATA_FLOAT = 101
DATA_INT = 99
DATA_COUNT64 = 64
DATA_COUNT = 32
DATA_STRING = 2
DATA_NONE = None

###############################################################################
# Constants for data DB ingestion
###############################################################################

MAX_KEYPAIR_LENGTH = 512

PattooDBrecord = collections.namedtuple(
    'PattooDBrecord', '''\
checksum data_index data_label data_source data_timestamp data_type \
data_value metadata''')

###############################################################################
# Constants for pattoo Agent API
###############################################################################

PATTOO_API_SITE_PREFIX = '/pattoo/api/v1'
PATTOO_API_AGENT_PREFIX = '{}/agent'.format(PATTOO_API_SITE_PREFIX)
PATTOO_API_AGENT_EXECUTABLE = 'pattoo-api-agentd'
PATTOO_API_AGENT_PROXY = '{}-gunicorn'.format(
    PATTOO_API_AGENT_EXECUTABLE)

###############################################################################
# Constants for standard agents
###############################################################################

# pattoo-agent-os constants
PATTOO_AGENT_OS_SPOKED_API_PREFIX = '/pattoo-agent-os'
PATTOO_AGENT_OS_SPOKED = 'pattoo-agent-os-spoked'
PATTOO_AGENT_OS_SPOKED_PROXY = '{}-gunicorn'.format(PATTOO_AGENT_OS_SPOKED)
PATTOO_AGENT_OS_AUTONOMOUSD = 'pattoo-agent-os-autonomousd'
PATTOO_AGENT_OS_HUBD = 'pattoo-agent-os-hubd'

# pattoo-snmp constants
PATTOO_AGENT_SNMPD = 'pattoo-agent-snmpd'

# pattoo-modbus-tcp constants
PATTOO_AGENT_MODBUSTCPD = 'pattoo-agent-modbustcpd'

# pattoo-modbus-tcp constants
PATTOO_AGENT_BACNETIPD = 'pattoo-agent-bacnetipd'



#!/usr/bin/env python3
"""Test the files module."""

# Standard imports
import unittest
import os
import sys

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
if EXEC_DIR.endswith('/pattoo-shared/tests/test_pattoo_shared') is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''\
This script is not installed in the "pattoo-shared/tests/test_pattoo_shared" \
directory. Please fix.''')
    sys.exit(2)

# Pattoo imports
from pattoo_shared import log
from tests.libraries.configuration import UnittestConfig


class Test_GetLog(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################
    def test_check_environment(self):
        """Testing function check_environment."""
        pass

    def test_logfile(self):
        """Testing function logfile."""
        pass

    def test_stdout(self):
        """Testing function stdout."""
        pass


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    message = 'PATTOOs are Jamaican Owls'
    code = 99

    def test_check_environment(self):
        """Testing function check_environment."""
        pass

    def test_log2console(self):
        """Testing function log2console."""
        # Test should not cause script to crash
        log.log2console(self.code, self.message)

    def test_log2die_safe(self):
        """Testing function log2die_safe."""
        # Test
        with self.assertRaises(SystemExit):
            log.log2die_safe(self.code, self.message)

    def test_log2warning(self):
        """Testing function log2warning."""
        # Test should not cause script to crash
        log.log2warning(self.code, self.message)

    def test_log2debug(self):
        """Testing function log2debug."""
        # Test should not cause script to crash
        log.log2debug(self.code, self.message)

    def test_log2info(self):
        """Testing function log2info."""
        # Test should not cause script to crash
        log.log2info(self.code, self.message)

    def test_log2see(self):
        """Testing function log2see."""
        # Test should not cause script to crash
        log.log2see(self.code, self.message)

    def test_log2die(self):
        """Testing function log2die."""
        # Test
        with self.assertRaises(SystemExit):
            log.log2die(self.code, self.message)

    def test__logit(self):
        """Testing function _logit."""
        pass

    def test__logger_file(self):
        """Testing function _logger_file."""
        pass

    def test__logger_stdout(self):
        """Testing function _logger_stdout."""
        pass

    def test__message(self):
        """Testing function _message."""
        # Test message for errors
        result = log._message(self.code, self.message, error=True)
        self.assertEqual(result.endswith(
            ' - ERROR - [99] PATTOOs are Jamaican Owls'), True)

        # Test message for non-errors
        result = log._message(self.code, self.message, error=False)
        self.assertEqual(result.endswith(
            ' - STATUS - [99] PATTOOs are Jamaican Owls'), True)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Class used to create the configuration file used for unittesting.

NOTE!! This script CANNOT import any pattoo-shared libraries. Doing so risks
libraries trying to access a configuration or configuration directory that
doesn't yet exist. This is especially important when running cloud based
automated tests such as 'Travis CI'.

"""

# Standard imports
from __future__ import print_function
import tempfile
import os
import yaml

# Pattoo imports
from pattoo_shared import log


class UnittestConfig(object):
    """Creates configuration for testing."""

    def __init__(self):
        """Initialize the class."""
        # Initialize GLOBAL variables
        config_suffix = '.pattoo-shared-unittests/config'
        self._config_directory = (
            '{}/{}'.format(os.environ['HOME'], config_suffix))

        # Make sure the environmental variables are OK
        _environment(self._config_directory)

        # Set global variables
        self._log_directory = tempfile.mkdtemp()
        self._cache_directory = tempfile.mkdtemp()
        self._daemon_directory = tempfile.mkdtemp()

        # Make sure the configuration directory is OK
        if os.path.isdir(self._config_directory) is False:
            os.makedirs(self._config_directory, mode=0o750, exist_ok=True)

        self._config = {
            'main': {
                'log_directory': self._log_directory,
                'log_level': 'debug',
                'cache_directory': self._cache_directory,
                'daemon_directory': self._daemon_directory,
                'polling_interval': 20
            },
            'pattoo-api-agentd': {
                'api_ip_address': '127.0.0.1',
                'api_ip_bind_port': 6060,
                'api_uses_https': False
            },
            'pattoo-agent-snmpd': {
                'oid_groups': [
                    {
                        'group_name': 'TEST',
                        'ip_devices': ['localhost'],
                        'oids': ['.1.3.6.1.2.1.2.2.1.10',
                                 '.1.3.6.1.2.1.2.2.1.16']
                    }
                ],
                'snmp_groups': [
                    {
                        'group_name': 'TEST',
                        'snmp_authpassword': None,
                        'snmp_authprotocol': None,
                        'snmp_community': 'public',
                        'snmp_port': 161,
                        'snmp_privpassword': None,
                        'snmp_privprotocol': None,
                        'snmp_secname': None,
                        'snmp_version': 2,
                        'ip_devices': ['localhost']
                    }
                ]
            },
            'pattoo-agent-os-spoked': {
                'listen_address': '127.0.0.1',
                'ip_bind_port': 5000
                },
            'pattoo-agent-os-hubd': {

                'ip_devices': [
                    {'ip_address': '127.0.0.1',
                     'ip_bind_port': 5000}]
                }
        }

    def create(self):
        """Create a good config and set the PATTOO_CONFIGDIR variable.

        Args:
            None

        Returns:
            self.config_directory: Directory where the config is placed

        """
        # Initialize key variables
        config_file = '{}/unittest_config.yaml'.format(self._config_directory)

        # Write good_config to file
        with open(config_file, 'w') as f_handle:
            yaml.dump(self._config, f_handle, default_flow_style=False)

        # Return
        return self._config_directory

    def cleanup(self):
        """Remove all residual directories.

        Args:
            None

        Returns:
            None

        """
        # Delete directories
        directories = [
            self._log_directory,
            self._cache_directory,
            self._daemon_directory,
            self._config_directory]
        for directory in directories:
            _delete_files(directory)


def _delete_files(directory):
    """Delete all files in directory."""
    # Cleanup files in temp directories
    filenames = [filename for filename in os.listdir(
        directory) if os.path.isfile(
            os.path.join(directory, filename))]

    # Get the full filepath for the cache file and remove filepath
    for filename in filenames:
        filepath = os.path.join(directory, filename)
        os.remove(filepath)

    # Remove directory after files are deleted.
    os.rmdir(directory)


def _environment(config_directory):
    """Make sure environmental variables are OK.

    Args:
        config_directory: Directory with the configuration

    Returns:
        None

    """
    # Create a message for the screen
    screen_message = ('''
The PATTOO_CONFIGDIR is set to the wrong directory. Run this command to do \
so:

$ export PATTOO_CONFIGDIR={}

Then run this command again.
'''.format(config_directory))

    # Make sure the PATTOO_CONFIGDIR environment variable is set
    if 'PATTOO_CONFIGDIR' not in os.environ:
        log.log2die_safe(1023, screen_message)

    # Make sure the PATTOO_CONFIGDIR environment variable is set correctly
    if os.environ['PATTOO_CONFIGDIR'] != config_directory:
        log.log2die_safe(1024, screen_message)

    # Update message
    screen_message = ('''{}

PATTOO_CONFIGDIR is incorrectly set to {}

'''.format(screen_message, os.environ['PATTOO_CONFIGDIR']))

    # Make sure the PATTOO_CONFIGDIR environment variable is set to unittest
    if 'unittest' not in os.environ['PATTOO_CONFIGDIR']:
        log_message = (
            'The PATTOO_CONFIGDIR is not set to a unittest directory')
        log.log2die_safe(1025, log_message)



#!/usr/bin/env python3
"""Test all the pattoo modules."""

from __future__ import print_function
import os
import inspect
import re
import sys
import collections


# Import pattoo libraries
from pattoo_shared import log


def check_source_code(root, minimum=0, maximum=0):
    """Get all the error codes used in pattoo related projects.

    Args:
        root: Root directory from which to search for error codes
        minimum: Minimum error code value to be found
        maximum: Maximum error code value to be found

    Returns:
        None

    """
    # Make sure minimum < maximum
    if minimum > maximum:
        log_message = ('''\
Minimum value {} is greater than {}. Please fix.\
'''.format(minimum, maximum))
        print(log_message)
        sys.exit(2)

    # Define where pattoo lives
    ignore_paths = [
        '/.git/', '/__pycache__/', '/_archive/', '/_deprecated/',
        '/pattoo-shared/build/', '/pattoo-shared/dist/', '.egg',
        '/docs/', '/daemon/', '/log/']
    error_functions = (
        'log2die_safe(', 'log2warning(',
        'log2debug(', 'log2live(', 'log2warn(', 'log2die(', 'log2quiet(',
        'log2info(', 'log2screen(', 'log2see(', r'.db_modify(', r'.db_query(',
        r'.db_commit(', r'.db_replace(', r'.db_add(', '.db_add_all(',
        r'.db_modify(', r'.db_query(')
    error_codes = []
    available_codes = []
    entries = 5

    # Get ready to ignore this script
    this_script = os.path.abspath(inspect.getfile(inspect.currentframe()))
    ignore_paths.append(this_script)

    # Compile regex to be used to find functions with error codes
    to_find = _wordlist_to_regex(error_functions)

    # Get a recursive listing of files
    python_files = _files(root, ignore_paths)

    # Read each file to find error codes
    for python_file in python_files:
        error_codes.extend(_codes(python_file, to_find))

    # Get duplicate codes
    _duplicates = [
        item for item, count in collections.Counter(
            error_codes).items() if count > 1]
    _duplicates.sort()
    if len(_duplicates) > entries:
        duplicates = '{} plus {} more...'.format(
            _duplicates[0:entries], len(_duplicates) - entries)
    else:
        duplicates = _duplicates

    # Determine whether codes are in range (MAX)
    code_max = max(error_codes)
    if int(code_max) > maximum:
        log_message = ('''\
Extremely large error code {} found. Must be less than {}. Please fix.\
'''.format(code_max, maximum))
        print(log_message)
        sys.exit(2)

    # Determine whether codes are in range (MIN)
    code_min = min(error_codes)
    if int(code_min) < minimum:
        log_message = ('''\
Extremely small error code {} found. Must be greater than {}. Please fix.\
'''.format(code_min, minimum))
        print(log_message)
        sys.exit(2)

    # Process error codes
    for next_code in range(min(error_codes), code_max):
        if next_code not in error_codes:
            available_codes.append(next_code)

    # Get available codes
    if bool(available_codes) is False:
        available_codes = list(
            range(max(error_codes), max(error_codes) + entries + 1))

    # Print report
    print('''
Pattoo Logging Error Code Summary Report
----------------------------------------
Starting Code              : {}
Ending Code                : {}
Duplicate Codes to Resolve : {}
Available Codes            : {}\
'''.format(min(error_codes),
           max(error_codes),
           duplicates,
           available_codes[0:entries]))

    # Exit with error if duplicate codes found
    if bool(duplicates) is True:
        print('''

ERROR: Duplicate error codes found. Please resolve.

''')
        sys.exit(1)

    # Exit with error if code values are out of range
    if (min(error_codes) < minimum) or (max(error_codes) > maximum):
        print('''

ERROR: Error codes values out of range {} to {}. Please resolve.

'''.format(minimum, maximum))
        sys.exit(1)



def _codes(filename, to_find):
    """Get a list of codes found in a file.

    Args:
        filename: Name of file
        to_find: Compiled list of functions to search for

    Returns:
        error_codes: List of error codes

    """
    # Initalize key variables
    digits = re.compile(r'^.*?(\d+).*?$')
    error_codes = []

    # Process file for codes
    with open(filename, 'r') as lines:
        # Read each line of the file
        for line in lines:
            # Ignore lines without a '(' in it
            if '(' not in line:
                continue

            # Ignore lines starting with comments
            if line.strip().startswith('#'):
                continue

            match_obj = to_find.search(line)
            if bool(match_obj) is True:
                # Search for digits in the arguments of the functions
                # in the error_functions
                components = line.split('(')
                arguments = ' '.join(components[1:])
                found = digits.match(arguments)
                if bool(found) is True:
                    # print('boo -', line)
                    error_codes.append(int(found.group(1)))

    # Exit with error if error codes are non numeric
    for _code in error_codes:
        if isinstance(_code, int) is False:
            print('''

ERROR: Non integer error code found. Please resolve.

'''.format(_code))
            sys.exit(1)

    # Return
    return error_codes


def _wordlist_to_regex(words):
    """Convert word list to a regex expression, escaping characters if needed.

    Args:
        words: List of words

    Returns:
        result: Compiled regex

    """
    escaped = map(re.escape, words)
    combined = '|'.join(sorted(escaped, key=len, reverse=True))
    result = re.compile(combined)
    return result


def _files(root, ignore_paths):
    """Get a recursive list of python files under a root directory.

    Args:
        root: Root directory
        ignore_paths: List of paths to ignore

    Returns:
        python_files: List of paths to python files

    """
    # Define where pattoo lives
    python_files = []

    # Get a recursive listing of files
    for directory, _, filenames in os.walk(root):
        for filename in filenames:
            file_path = os.path.join(directory, filename)

            # Ignore pre-defined file extensions
            if file_path.endswith('.py') is False:
                continue

            # Ignore pre-defined paths
            ignored = False
            for ignore_path in ignore_paths:
                if ignore_path in file_path:
                    ignored = True
            if ignored is False:
                python_files.append(file_path)

    # Return
    return python_files



#!/usr/bin/env python3
"""Test the daemon module."""

# Standard imports
import unittest
import os
import sys


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
if EXEC_DIR.endswith('/pattoo-shared/tests/test_pattoo_shared') is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''\
This script is not installed in the "pattoo-shared/tests/test_pattoo_shared" \
directory. Please fix.''')
    sys.exit(2)

# Pattoo imports
from pattoo_shared import daemon
from pattoo_shared.configuration import Config
from tests.libraries.configuration import UnittestConfig


class TestDaemon(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing function __init__."""
        pass

    def test__daemonize(self):
        """Testing function _daemonize."""
        pass

    def test_delpid(self):
        """Testing function delpid."""
        pass

    def test_dellock(self):
        """Testing function dellock."""
        pass

    def test_start(self):
        """Testing function start."""
        pass

    def test_force(self):
        """Testing function force."""
        pass

    def test_stop(self):
        """Testing function stop."""
        pass

    def test_restart(self):
        """Testing function restart."""
        pass

    def test_status(self):
        """Testing function status."""
        pass

    def test_run(self):
        """Testing function run."""
        pass


class Test_Directory(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing function __init__."""
        # Test
        directory = daemon._Directory()
        config = Config()
        expected = config.daemon_directory()
        result = directory._root
        self.assertEqual(result, expected)

    def test_pid(self):
        """Testing function pid."""
        # Test
        directory = daemon._Directory()
        config = Config()
        expected = '{}/pid'.format(config.daemon_directory())
        result = directory.pid()
        self.assertEqual(result, expected)

    def test_lock(self):
        """Testing function lock."""
        # Test
        directory = daemon._Directory()
        config = Config()
        expected = '{}/lock'.format(config.daemon_directory())
        result = directory.lock()
        self.assertEqual(result, expected)

    def test_agent_id(self):
        """Testing function agent_id."""
        # Test
        directory = daemon._Directory()
        config = Config()
        expected = '{}/agent_id'.format(config.daemon_directory())
        result = directory.agent_id()
        self.assertEqual(result, expected)


class Test_File(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    prefix = 'unittest'
    agent_hostname = 'pattoo_host'

    def test___init__(self):
        """Testing function __init__."""
        pass

    def test_pid(self):
        """Testing function pid."""
        # Test
        filename = daemon._File()
        result = filename.pid(self.prefix)
        self.assertTrue(os.path.isdir(os.path.dirname(result)))

    def test_lock(self):
        """Testing function lock."""
        # Test
        filename = daemon._File()
        result = filename.lock(self.prefix)
        self.assertTrue(os.path.isdir(os.path.dirname(result)))

    def test_agent_id(self):
        """Testing function agent_id."""
        filename = daemon._File()
        result = filename.agent_id(self.prefix, self.agent_hostname)
        self.assertTrue(os.path.isdir(os.path.dirname(result)))


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    prefix = 'unittest'
    agent_hostname = 'pattoo_host'

    def test_pid_file(self):
        """Testing function pid_file."""
        # Test
        filename = daemon._File()
        expected = filename.pid(self.prefix)
        result = daemon.pid_file(self.prefix)
        self.assertEqual(result, expected)

    def test_lock_file(self):
        """Testing function lock_file."""
        # Test
        filename = daemon._File()
        expected = filename.lock(self.prefix)
        result = daemon.lock_file(self.prefix)
        self.assertEqual(result, expected)

    def test_agent_id_file(self):
        """Testing function agent_id_file."""
        # Test
        filename = daemon._File()
        expected = filename.agent_id(self.prefix, self.agent_hostname)
        result = daemon.agent_id_file(self.prefix, self.agent_hostname)
        self.assertEqual(result, expected)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Test the daemon module."""

# Standard imports
import unittest
import os
import sys
import multiprocessing
from random import random


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
if EXEC_DIR.endswith('/pattoo-shared/tests/test_pattoo_shared') is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''\
This script is not installed in the "pattoo-shared/tests/test_pattoo_shared" \
directory. Please fix.''')
    sys.exit(2)

# Pattoo imports
from pattoo_shared import agent, daemon
from pattoo_shared.configuration import Config
from pattoo_shared.variables import AgentAPIVariable
from tests.libraries.configuration import UnittestConfig


class TestAgent(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method / function __init__."""
        # Initialize key variables
        parent = 'parent'
        child = 'child'

        # Test - No Child
        tester = agent.Agent(parent)
        self.assertEqual(tester.parent, parent)
        expected = daemon.pid_file(parent)
        self.assertEqual(tester.pidfile_parent, expected)
        expected = daemon.lock_file(parent)
        self.assertEqual(tester.lockfile_parent, expected)
        expected = daemon.pid_file(None)
        self.assertEqual(tester._pidfile_child, expected)

        # Test - With Child
        tester = agent.Agent(parent, child=child)
        self.assertEqual(tester.parent, parent)
        expected = daemon.pid_file(parent)
        self.assertEqual(tester.pidfile_parent, expected)
        expected = daemon.lock_file(parent)
        self.assertEqual(tester.lockfile_parent, expected)
        expected = daemon.pid_file(child)
        self.assertEqual(tester._pidfile_child, expected)

    def test_name(self):
        """Testing method / function name."""
        # Initialize key variables
        parent = 'parent'

        # Test
        tester = agent.Agent(parent)
        self.assertEqual(tester.name(), parent)

    def test_query(self):
        """Testing method / function query."""
        pass


class TestAgentDaemon(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method / function __init__."""
        pass

    def test_run(self):
        """Testing method / function run."""
        pass


class TestAgentCLI(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method / function __init__."""
        pass

    def test_process(self):
        """Testing method / function process."""
        pass

    def test_control(self):
        """Testing method / function control."""
        pass


class TestAgentAPI(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method / function __init__."""
        pass

    def test_query(self):
        """Testing method / function query."""
        pass


class Test_StandaloneApplication(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method / function __init__."""
        pass

    def test_load_config(self):
        """Testing method / function load_config."""
        pass

    def test_load(self):
        """Testing method / function load."""
        pass


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test__number_of_workers(self):
        """Testing method / function _number_of_workers."""
        # Test
        expected = (multiprocessing.cpu_count() * 2) + 1
        result = agent._number_of_workers()
        self.assertEqual(result, expected)

    def test__ip_binding(self):
        """Testing method / function _ip_binding."""
        # Test with Hostname
        listen_address = 'localhost'
        ip_bind_port = '1000'
        agent_api_variable = AgentAPIVariable(
            ip_bind_port=ip_bind_port, listen_address=listen_address)
        result = agent._ip_binding(agent_api_variable)
        expected = 'localhost:1000'
        self.assertEqual(result, expected)

        # Test with IPv4
        listen_address = '1.2.3.4'
        ip_bind_port = '5678'
        agent_api_variable = AgentAPIVariable(
            ip_bind_port=ip_bind_port, listen_address=listen_address)
        result = agent._ip_binding(agent_api_variable)
        expected = '1.2.3.4:5678'
        self.assertEqual(result, expected)

        # Test with IPv6
        listen_address = '1::1'
        ip_bind_port = '91011'
        agent_api_variable = AgentAPIVariable(
            ip_bind_port=ip_bind_port, listen_address=listen_address)
        result = agent._ip_binding(agent_api_variable)
        expected = '[1::1]:91011'
        self.assertEqual(result, expected)

    def test_get_agent_id(self):
        """Testing method / function get_agent_id."""
        # Test. Agent_id shouldn't change
        agent_name = random()
        agent_hostname = random()
        expected = agent.get_agent_id(agent_name, agent_hostname)
        result = agent.get_agent_id(agent_name, agent_hostname)
        self.assertEqual(result, expected)

    def test__generate_agent_id(self):
        """Testing method / function _generate_agent_id."""
        pass


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



"""Module for classes that format variables."""

# Standard imports
from time import time

# pattoo imports
from pattoo_shared import data
from .constants import (
    DATA_INT, DATA_FLOAT, DATA_COUNT64, DATA_COUNT, DATA_STRING, DATA_NONE)


class DataPointMeta(object):
    """Metadata related to a DataPoint."""

    def __init__(self, key, value):
        """Initialize the class.

        Args:
            key: Metadata key
            value: Metadata value

        Returns:
            None

        Variables:
            self.key: Metadata key
            self.value: Metadata value
            self.valid: True if valid

        """
        # Initialize variables
        self.key = None
        self.value = None

        # Set variables
        if isinstance(key, (str, int, float)) is True and key is not True and (
                key is not False):
            self.key = str(key)
        if isinstance(value, (str, int, float)) is True and (
                value is not True and value is not False):
            self.value = str(value)

        self.valid = False not in [
            bool(self.key),
            self.key is not True,
            self.key is not False,
            self.key is not None,
            bool(self.value),
            self.value is not True,
            self.value is not False,
            self.value is not None,
            ]

    def __repr__(self):
        """Return a representation of the attributes of the class.

        Args:
            None

        Returns:
            result: String representation.

        """
        # Create a printable variation of the value
        result = ('<{0} key={1}, value={2}>'.format(
            self.__class__.__name__, repr(self.key), repr(self.value))
        )
        return result


class DataPoint(object):
    """Variable representation for data retreived from a device.

    Stores individual datapoints polled by pattoo agents

    """

    def __init__(self, value, data_label=None,
                 data_index=0, data_type=DATA_INT):
        """Initialize the class.

        Args:
            value: Value of data for a given data_index and data_label
            data_label:
                A label that describes the type of data being polled.
                This value must be unique to the agent polling data. (eg. SNMP
                ifDescr OID '1.3.6.1.2.1.2.2.1.2', Linux 'load_average').
                Different agents can use the same data_labels.
            data_index:
                Unique index value of data point. The combination of data_index
                and data_label must be unique for any polled device. (eg. SNMP
                IfIndex value, Linux '5' for the 5 minute load average). If
                there is only one instance of the data being tracked by
                data_label, then the data_index should be 0.
            data_type: This MUST be one of the types listed in constants.py

        Returns:
            None

        Variables:
            self.data_timestamp: Integer of epoch milliseconds
            self.valid: True if the object has a valid data_type
            self.checksum: Hash of self.data_label, self.data_index and
                self.data_type to ensure uniqueness when assigned to a device.

        """
        # Initialize variables
        self.data_label = data_label
        self.data_index = data_index
        self.data_value = value
        self.data_type = data_type
        self.data_timestamp = int(time() * 1000)
        self.metadata = {}

        # False validity if value is not of the right type
        self.valid = False not in [
            data_type in [DATA_INT, DATA_FLOAT, DATA_COUNT64, DATA_COUNT,
                          DATA_STRING, DATA_NONE],
            data_type is not False,
            data_type is not True,
            data_type is not None
        ]
        if False not in [
                data_type in [DATA_INT, DATA_FLOAT, DATA_COUNT64, DATA_COUNT],
                self.valid is True, data.is_numeric(value) is False]:
            self.valid = False

        # Convert floatable strings to float, and integers to ints
        if False not in [
                self.valid is True,
                data.is_numeric(value) is True,
                isinstance(value, str) is True]:
            if data_type in [DATA_FLOAT, DATA_COUNT64, DATA_COUNT]:
                self.data_value = float(value)
            elif data_type in [DATA_INT]:
                self.data_value = int(float(value))

        # Convert strings to string
        if data_type in [DATA_STRING]:
            self.data_value = str(value)

        # Create checksum
        self.checksum = self._checksum()

    def __repr__(self):
        """Return a representation of the attributes of the class.

        Args:
            None

        Returns:
            result: String representation.

        """
        # Create a printable variation of the value
        printable_value = _strip_non_printable(self.data_value)
        result = ('''\
<{0} data_value={1}, data_label={2}, data_index={3}, data_type={4}, \
data_timestamp={6}, valid={5}>\
'''.format(self.__class__.__name__,
           repr(printable_value), repr(self.data_label),
           repr(self.data_index), repr(self.data_type),
           repr(self.valid), repr(self.data_timestamp))
        )
        return result

    def add(self, items):
        """Add DataPointMeta to the internal self.metadata list.

        Args:
            items: A DataPointMeta object list

        Returns:
            None

        """
        # Ensure there is a list of objects
        if isinstance(items, list) is False:
            items = [items]

        # Only append approved data types
        for item in items:
            if isinstance(item, DataPointMeta) is True:
                # Ignore invalid values
                if item.valid is False:
                    continue

                # Process
                self.metadata[item.key] = item.value

    def _checksum(self):
        """Calculate a checksum for the DataPoint.

        Args:
            None

        Returns:
            checksum: Hash.

        """
        # Initialize key variables
        seed = '{}{}{}'.format(
            self.data_label, self.data_type, self.data_index)
        checksum = data.hashstring(seed)

        # Generate checksum
        for key, value in sorted(self.metadata.items()):
            checksum = data.hashstring('{}{}{}'.format(checksum, key, value))
        return checksum


class DeviceDataPoints(object):
    """Object defining a list of DataPoint objects.

    Stores DataPoints polled from a specific ip_device.

    """

    def __init__(self, device, device_type=None):
        """Initialize the class.

        Args:
            device: Device polled to get the DataPoint objects
            device_type: Integer value identifying the type of device.
                Device_types are specific to the agent_program name. The
                intention is to make the combination of device_type and
                agent_program be the combined key in doing datapoint
                description lookups.

        Returns:
            None

        Variables:
            self.data: List of DataPoints retrieved from the device
            self.valid: True if the object is populated with DataPoints

        """
        # Initialize key variables
        self.data = []
        self.device = device
        self.valid = False
        self.device_type = None
        self._checksums = []
        self.device_type = _device_type(device_type)

    def __repr__(self):
        """Return a representation of the attributes of the class.

        Args:
            None

        Returns:
            result: String representation.

        """
        # Create a printable variation of the value
        result = (
            '<{0} device={1}, device_type={4}, valid={2}, data={3}'
            ''.format(
                self.__class__.__name__,
                repr(self.device), repr(self.valid), repr(self.data),
                repr(self.device_type)
            )
        )
        return result

    def add(self, items):
        """Append DataPoint to the internal self.data list.

        Args:
            items: A DataPoint object list

        Returns:
            None

        """
        # Ensure there is a list of objects
        if isinstance(items, list) is False:
            items = [items]

        # Only add DataPoint objects that are not duplicated
        for item in items:
            if isinstance(item, DataPoint) is True:
                if item.checksum not in self._checksums:
                    self.data.append(item)
                    self._checksums.append(item.checksum)

                # Set object as being.valid
                self.valid = False not in [bool(self.data), bool(self.device)]


class DeviceGateway(object):
    """Object defining a list of DeviceDataPoints objects.

    Stores DeviceDataPoints polled from a specific ip_device.

    """

    def __init__(self, device):
        """Initialize the class.

        Args:
            device: Device polled to get the DeviceDataPoints objects

        Returns:
            None

        Variables:
            self.data: List of DeviceDataPoints retrieved from the device
            self.valid: True if the object has assigned DeviceDataPoints

        """
        # Initialize key variables
        self.data = []
        self.device = device
        self.valid = False

    def __repr__(self):
        """Return a representation of the attributes of the class.

        Args:
            None

        Returns:
            result: String representation.

        """
        # Create a printable variation of the value
        result = (
            '<{0} device={1}, valid={2}, data={3}>'
            ''.format(
                self.__class__.__name__,
                repr(self.device), repr(self.valid), repr(self.data)
            )
        )
        return result

    def add(self, items):
        """Add DeviceDataPoints to the internal self.data list.

        Args:
            items: A DeviceDataPoints object list

        Returns:
            None

        """
        # Ensure there is a list of objects
        if isinstance(items, list) is False:
            items = [items]

        # Only append approved data types
        for item in items:
            if isinstance(item, DeviceDataPoints) is True:
                # Ignore invalid values
                if item.valid is False:
                    continue

                # Process
                self.data.append(item)

                # Set object as being valid
                self.valid = False not in [
                    bool(self.data), bool(self.device)]


class AgentPolledData(object):
    """Object defining data received from / sent by Agent.

    Only AgentPolledData objects can be submitted to the pattoo server through
    phttp.Post()

    """

    def __init__(self, agent_id, agent_program, agent_hostname,
                 polling_interval):
        """Initialize the class.

        Args:
            agent_id: Agent ID
            agent_program: Name of agent program collecting the data
            agent_hostname: Hostname on which the agent ran
            polling_interval: Polling interval used to collect the data

        Returns:
            None

        Variables:
            self.data: List of DeviceGateway objects created by polling
            self.valid: True if the object contains DeviceGateway objects

        """
        # Initialize key variables
        self.agent_id = agent_id
        self.agent_program = agent_program
        self.agent_hostname = agent_hostname
        self.agent_timestamp = int(time() * 1000)
        self.polling_interval = polling_interval
        self.data = []
        self.valid = False

    def __repr__(self):
        """Return a representation of the attributes of the class.

        Args:
            None

        Returns:
            result: String representation.

        """
        # Return
        result = ('''\
<{0} agent_id={1} agent_program={2}, agent_hostname={3}, timestamp={4} \
polling_interval={5}, valid={6}>\
'''.format(self.__class__.__name__, repr(self.agent_id),
           repr(self.agent_program), repr(self.agent_hostname),
           repr(self.agent_timestamp), repr(self.polling_interval),
           repr(self.valid)))
        return result

    def add(self, items):
        """Append DeviceGateway to the internal self.data list.

        Args:
            items: A DeviceGateway object list

        Returns:
            None

        """
        # Do nothing if not a list
        if isinstance(items, list) is False:
            items = [items]

        # Only append approved data types
        for item in items:
            # Only append approved data types
            if isinstance(item, DeviceGateway) is True:
                # Ignore invalid values
                if item.valid is False:
                    continue

                # Process
                self.data.append(item)

                # Set object as being.valid
                self.valid = False not in [
                    bool(self.agent_id), bool(self.agent_program),
                    bool(self.agent_hostname), bool(self.polling_interval),
                    bool(self.data)]


class AgentAPIVariable(object):
    """Variable representation for data required by the AgentAPI."""

    def __init__(self, ip_bind_port=6000, listen_address='0.0.0.0'):
        """Initialize the class.

        Args:
            ip_bind_port: ip_bind_port
            listen_address: TCP/IP address on which the API is listening.

        Returns:
            None

        """
        # Initialize variables
        self.ip_bind_port = ip_bind_port
        self.listen_address = listen_address

    def __repr__(self):
        """Return a representation of the attributes of the class.

        Args:
            None

        Returns:
            result: String representation.

        """
        result = ('''\
<{0} ip_bind_port={1}, listen_address={2}>\
'''.format(self.__class__.__name__,
           repr(self.ip_bind_port),
           repr(self.listen_address)
           )
        )
        return result


class PollingTarget(object):
    """Object used to track data to be polled."""

    def __init__(self, address=None, multiplier=1):
        """Initialize the class.

        Args:
            address: Address to poll
            multiplier: Multiplier to use when polled

        Returns:
            None

        """
        # Initialize variables
        self.address = address
        if data.is_numeric(multiplier) is True:
            self.multiplier = multiplier
        else:
            self.multiplier = 1
        self.valid = address is not None

        # Create checksum
        seed = '{}{}'.format(address, multiplier)
        self.checksum = data.hashstring(seed)

    def __repr__(self):
        """Return a representation of the attributes of the class.

        Args:
            None

        Returns:
            result: String representation.

        """
        result = ('''\
<{0} address={1}, multiplier={2}>\
'''.format(self.__class__.__name__,
           repr(self.address),
           repr(self.multiplier)
           ))
        return result


class DevicePollingTargets(object):
    """Object defining a list of PollingTarget objects.

    Stores PollingTargets polled from a specific ip_device.

    """

    def __init__(self, device, device_type=None):
        """Initialize the class.

        Args:
            device: Device polled to get the PollingTarget objects
            device_type: Integer value identifying the type of device.
                Device_types are specific to the agent_program name. The
                intention is to make the combination of device_type and
                agent_program be the combined key in doing datapoint
                description lookups.

        Returns:
            None

        Variables:
            self.data: List of PollingTargets retrieved from the device
            self.device: Name of device from which the data was received
            self.device_type: Type of device
            self.valid: True if the object is populated with PollingTargets

        """
        # Initialize key variables
        self.data = []
        self.device = device
        self.valid = False
        self._checksums = []
        self.device_type = _device_type(device_type)

    def __repr__(self):
        """Return a representation of the attributes of the class.

        Args:
            None

        Returns:
            result: String representation.

        """
        # Create a printable variation of the value
        result = (
            '<{0} device={1}, device_type={4}, valid={2}, data={3}>'
            ''.format(
                self.__class__.__name__,
                repr(self.device), repr(self.valid), repr(self.data),
                repr(self.device_type)
            )
        )
        return result

    def add(self, items):
        """Append PollingTarget to the internal self.data list.

        Args:
            items: A PollingTarget object list

        Returns:
            None

        """
        # Ensure there is a list of objects
        if isinstance(items, list) is False:
            items = [items]

        # Only add PollingTarget objects that are not duplicated
        for item in items:
            if isinstance(item, PollingTarget) is True:
                # Ignore invalid values
                if item.valid is False:
                    continue

                # Add data to the list
                if item.checksum not in self._checksums:
                    self.data.append(item)

                # Set object as being.valid
                self.valid = False not in [bool(self.data), bool(self.device)]


def _strip_non_printable(value):
    """Strip non printable characters.

    Removes any non-printable characters and adds an indicator to the string
    when binary characters are found.

    Args:
        value: the value that you wish to strip

    Returns:
        printable_value: Printable string

    """
    # Initialize key variables
    printable_value = ''

    if isinstance(value, str) is False:
        printable_value = value
    else:
        # Filter all non-printable characters
        # (note that we must use join to account for the fact that Python 3
        # returns a generator)
        printable_value = ''.join(
            [x for x in value if x.isprintable() is True])
        if printable_value != value:
            if bool(printable_value) is True:
                printable_value = '{} '.format(printable_value)
            printable_value = '{}(contains binary)'.format(printable_value)

    # Return
    return printable_value


def _device_type(device_type):
    """Create a standardized device type.

    Args:
        device_type: Type of device

    Returns:
        result: Standardized device_type

    """
    # Initialize key variables
    valid = True

    # Set the device_type
    if device_type is None or device_type is True or device_type is False:
        valid = False
    if valid is True:
        try:
            result = abs(int(device_type))
        except:
            result = None
    else:
        result = None
    return result



#!/usr/bin/env python3
"""Pattoo helper for the Linux _data.

Description:

    Uses Python2 to be compatible with most Linux systems


"""
# Standard libraries
import hashlib

# pattoo imports
from .constants import DATA_INT, DATA_FLOAT, DATA_COUNT64, DATA_COUNT


def hashstring(string, sha=256, utf8=False):
    """Create a UTF encoded SHA hash string.

    Args:
        string: String to hash
        length: Length of SHA hash
        utf8: Return utf8 encoded string if true

    Returns:
        result: Result of hash

    """
    # Initialize key variables
    listing = [1, 224, 384, 256, 512]

    # Select SHA type
    if sha in listing:
        index = listing.index(sha)
        if listing[index] == 1:
            hasher = hashlib.sha1()
        elif listing[index] == 224:
            hasher = hashlib.sha224()
        elif listing[index] == 384:
            hasher = hashlib.sha384()
        elif listing[index] == 512:
            hasher = hashlib.sha512()
        else:
            hasher = hashlib.sha256()

    # Encode the string
    hasher.update(bytes(string.encode()))
    device_hash = hasher.hexdigest()
    if utf8 is True:
        result = device_hash.encode()
    else:
        result = device_hash

    # Return
    return result


def is_numeric(val):
    """Check if argument is a number.

    Args:
        val: String to check

    Returns:
        True if a number

    """
    # Try edge case
    if val is True:
        return False
    if val is False:
        return False
    if val is None:
        return False

    # Try conversions
    try:
        float(val)
        return True
    except ValueError:
        return False
    except TypeError:
        return False
    except:
        return False


def is_data_type_numeric(data_type):
    """Check if data_type argument is a number.

    Args:
        data_type: Value to check

    Returns:
        result: True if a number.

    """
    # Try edge case
    result = False not in [
        data_type in [DATA_INT, DATA_FLOAT, DATA_COUNT64, DATA_COUNT],
        is_numeric(data_type) is True]
    return result



#!/usr/bin/env python3
"""Pattoo Data Converter."""

# Standard libraries
from collections import defaultdict
from copy import deepcopy
from operator import attrgetter
import json

# Pattoo libraries
from .variables import (
    DataPointMeta, DataPoint, DeviceDataPoints, DeviceGateway, AgentPolledData)
from .constants import (
    PattooDBrecord,
    DATA_FLOAT, DATA_INT, DATA_COUNT64, DATA_COUNT, DATA_STRING, DATA_NONE)
from pattoo_shared import times
from pattoo_shared import data as lib_data


class ConvertAgentPolledData(object):
    """Converts AgentPolledData object to a standardized dict."""

    def __init__(self, agentdata):
        """Initialize the class.

        Args:
            agentdata: AgentPolledData object of data polled by agent

        Returns:
            None

        """
        # Initialize key variables
        self._data = defaultdict(lambda: defaultdict(dict))
        self._gateway_data = agentdata.data
        self._data['agent_timestamp'] = agentdata.agent_timestamp
        self._data['polling_interval'] = agentdata.polling_interval
        self._data['agent_id'] = agentdata.agent_id
        self._data['agent_program'] = agentdata.agent_program
        self._data['agent_hostname'] = agentdata.agent_hostname
        self._data['gateways'] = self._process()

    def _process(self):
        """Process.

        Args:
            None

        Returns:
            result: Data required

        """
        # Intitialize key variables
        result = {}

        if isinstance(self._gateway_data, list) is True:
            for gwd in self._gateway_data:
                if isinstance(gwd, DeviceGateway) is True:
                    if bool(gwd.valid) is True:
                        # Get information from data
                        gwd_dict = _capd_gwd2dict(gwd)
                        for gateway, values in gwd_dict.items():
                            result[gateway] = values

        # Return
        return result

    def data(self):
        """Return that that should be posted.

        Args:
            None

        Returns:
            None

        """
        # Return
        return self._data


def convert(_data=None):
    """Convert agent cache data to AgentPolledData object.

    Args:
        _data: Agent data dict

    Returns:
        agentdata: AgentPolledData object

    """
    # Initialize key variables
    agent_id = None
    agent_program = None
    agent_hostname = None
    polling_interval = None

    # Get values to instantiate an AgentPolledData object
    (agent_id, agent_program, agent_hostname, polling_interval,
     polled_data, agent_valid) = _valid_agent(_data)
    if agent_valid is False:
        return None
    agentdata = AgentPolledData(
        agent_id, agent_program, agent_hostname, polling_interval)

    # Iterate through devices polled by the agent
    for gateway, gw_dict in sorted(polled_data.items()):
        # Create an populate the DeviceGateway object
        gwd = DeviceGateway(gateway)
        for device, devicedata in sorted(gw_dict['devices'].items()):
            # Append the DeviceDataPoints to the DeviceGateway object
            ddv = _create_ddv(device, devicedata)
            if ddv.valid is True:
                gwd.add(ddv)

        # Append the DeviceGateway to the AgentPolledData object
        if gwd.valid is True:
            agentdata.add(gwd)

    # Return
    if agentdata.valid is False:
        return None
    else:
        return agentdata


def extract(agentdata):
    """Ingest data.

    Args:
        agentdata: AgentPolledData object

    Returns:
        rows: List of named tuples containing data

    """
    # Initialize key variables
    rows = []

    # Only process valid data
    if isinstance(agentdata, AgentPolledData) is True:
        # Return if invalid data
        if bool(agentdata.valid) is False:
            return []

        # Assign agent values
        agent_id = agentdata.agent_id
        agent_program = agentdata.agent_program
        agent_hostname = agentdata.agent_hostname
        polling_interval = agentdata.polling_interval
        agent_program = agentdata.agent_program
        agent_timestamp = agentdata.agent_timestamp

        # Cycle through the data
        for gwd in agentdata.data:
            # Ignore bad data
            if gwd.valid is False:
                continue

            # Get gateway from which data came
            gateway = gwd.device

            for ddv in gwd.data:
                # Ignore bad data
                if ddv.valid is False:
                    continue

                # Get data
                device = ddv.device
                device_type = ddv.device_type
                for _dv in ddv.data:
                    # Assign values to DataPoints
                    metadata = {
                        'agent_id': agent_id,
                        'agent_program': agent_program,
                        'agent_hostname': agent_hostname,
                        'agent_timestamp': agent_timestamp,
                        'polling_interval': polling_interval,
                        'gateway': gateway,
                        'device': device,
                        'device_type': device_type
                    }
                    for key, value in metadata.items():
                        _dv.add(DataPointMeta(key, value))
                    rows.append(_dv)

    # Return
    return rows


def _add_checksum(row):
    """Determine the validity of the Agent's data.

    Args:
        row: PattooDBrecord object

    Returns:
        checksum: Calculated checksum

    """
    json_metadata = json.dumps(row.metadata, sort_keys=True)
    checksum = lib_data.hashstring('''{}{}{}{}{}{}{}{}{}{}\
'''.format(row.agent_id, row.agent_program, row.agent_hostname, row.gateway,
           row.device, row.data_label, row.data_index, row.data_type,
           row.device_type, json_metadata))

    result = PattooDBrecord(
        agent_id=row.agent_id,
        agent_program=row.agent_program,
        agent_hostname=row.agent_hostname,
        data_type=row.data_type,
        polling_interval=row.polling_interval,
        gateway=row.gateway,
        device=row.device,
        device_type=row.device_type,
        metadata=row.metadata,
        data_label=row.data_label,
        data_index=row.data_index,
        checksum=checksum,
        value=row.value,
        agent_timestamp=row.agent_timestamp,
        timestamp=row.timestamp)
    return result


def _valid_agent(_data):
    """Determine the validity of the Agent's data.

    Args:
        _data: Agent data dict

    Returns:
        result: Tuple of (
            agent_id, agent_program, agent_hostname, polled_data, agent_valid)

    """
    # Initialize key variables
    agent_id = None
    agent_program = None
    agent_hostname = None
    polling_interval = None
    polled_data = None
    agent_valid = False

    # Verify values
    if isinstance(_data, dict) is True:
        if 'agent_id' in _data:
            agent_id = _data['agent_id']
        if 'agent_program' in _data:
            agent_program = _data['agent_program']
        if 'agent_hostname' in _data:
            agent_hostname = _data['agent_hostname']
        if 'polling_interval' in _data:
            if isinstance(_data['polling_interval'], int) is True:
                polling_interval = _data['polling_interval']
        if 'gateways' in _data:
            if isinstance(_data['gateways'], dict) is True:
                polled_data = deepcopy(_data['gateways'])

    # Determine validity
    agent_valid = False not in [
        bool(agent_id), bool(agent_program),
        bool(agent_hostname),
        bool(polling_interval), bool(polled_data)]

    # Return
    result = (
        agent_id, agent_program, agent_hostname,
        polling_interval, polled_data, agent_valid)
    return result


def _create_ddv(device, devicedata):
    """Create a DeviceDataPoints object from Agent data.

    Args:
        device: Device polled by agent
        devicedata: Data polled from device by agent

    Returns:
        ddv: DeviceDataPoints object

    """
    # Initialize key variables
    ddv = DeviceDataPoints(device, device_type=None)

    # Ignore invalid data
    if isinstance(devicedata, dict) is True:
        if ('datapoints' in devicedata) and ('device_type' in devicedata):
            # Create a valid DeviceDataPoint object
            device_type = devicedata['device_type']
            ddv = DeviceDataPoints(device, device_type=device_type)

            # Iterate through the expected data_labels in the dict
            datapoint_dict = devicedata['datapoints']
            for data_label, label_dict in sorted(datapoint_dict.items()):
                # Ignore invalid data
                if isinstance(label_dict, dict) is False:
                    continue

                # Validate the presence of required keys, then process
                if ('data' in label_dict) and ('data_type' in label_dict):
                    # Skip invalid data formats
                    if isinstance(label_dict['data'], list) is False:
                        continue

                    # Add to the DeviceDataPoints
                    datapoints = _create_datapoints(data_label, label_dict)
                    ddv.add(datapoints)

    # Return
    return ddv


def _create_datapoints(data_label, label_dict):
    """Create a valid list of DataPoints for a specific label.

    Args:
        data_label: Label for data
        label_dict: Dict of data represented by the data_label

    Returns:
        datapoints: List of DataPoint objects

    """
    # Initialize key variables
    datapoints = []
    data_type = label_dict['data_type']
    found_type = False

    # Skip invalid types
    for next_type in [
            DATA_FLOAT, DATA_INT, DATA_COUNT64, DATA_COUNT,
            DATA_STRING, DATA_NONE]:
        if (data_type == next_type) and (data_type is not True) and (
                data_type is not False):
            found_type = True
    if found_type is False:
        return []

    # Add the data to the DeviceDataPoints
    for item in label_dict['data']:
        if isinstance(item, list) is True:
            if len(item) == 2:
                data_index = item[0]
                value = item[1]

                # Skip invalid numerical data
                if data_type not in (DATA_STRING, DATA_NONE):
                    try:
                        float(value)
                    except:
                        continue

                # Update DataPoint with valid data
                datapoint = DataPoint(
                    value=value,
                    data_label=data_label,
                    data_index=data_index,
                    data_type=label_dict['data_type'])
                datapoints.append(datapoint)

    # Return
    return datapoints


def _capd_gwd2dict(gwd):
    """Create dict representation of DeviceGateway object.

    Args:
        gwd: DeviceGateway object

    Returns:
        result: Representation of DeviceGateway as a dict

    """
    # Intitialize key variables
    result = {}

    # Verify data type
    if isinstance(gwd, DeviceGateway) is True:
        if bool(gwd.valid) is True:
            # Get information from data
            gateway = gwd.device

            # Analyze each DeviceDataPoints  object in data
            gwd_data_dict = {}
            for ddv in gwd.data:
                ddv_data_dict = _capd_ddv2dict(ddv)
                for remote_device, remote_data in sorted(
                        ddv_data_dict.items()):
                    if bool(remote_data) is True and (
                            isinstance(remote_data, dict) is True):
                        gwd_data_dict[remote_device] = remote_data

            # Update the result
            if bool(gwd_data_dict) is True:
                result[gateway] = {
                    'devices': gwd_data_dict
                }

    # Return
    return result


def _capd_ddv2dict(ddv):
    """Create dict representation of DeviceDataPoints object.

    Args:
        ddv: DeviceDataPoints object

    Returns:
        result: Representation of DeviceDataPoints as a dict

    """
    # Intitialize key variables
    result = {}

    # Verify data type
    if isinstance(ddv, DeviceDataPoints) is True:
        if bool(ddv.valid) is True:
            # Get information from data
            device = ddv.device
            device_type = ddv.device_type

            # Pre-populate the result with empty dicts
            datapoints = {}

            # Analyze each DataPoint for the ddv
            for _dvar in ddv.data:
                # Add keys if not already there
                if _dvar.data_label not in datapoints:
                    datapoints[_dvar.data_label] = {}

                # Assign data values to result
                data_tuple = (_dvar.data_index, _dvar.value)
                if 'data' in datapoints[_dvar.data_label]:
                    datapoints[_dvar.data_label][
                        'data'].append(data_tuple)
                else:
                    datapoints[_dvar.data_label][
                        'data_type'] = _dvar.data_type
                    datapoints[_dvar.data_label][
                        'data'] = [data_tuple]

            # Create a dict specific to the the device
            device_dict = {
                'datapoints': datapoints,
                'device_type': device_type
            }

            # Add it all to the result
            result[device] = device_dict

    # Return
    return result



#!/usr/bin/env python3
"""Test the phttp module."""

# Standard imports
import unittest
import os
import sys


# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
if EXEC_DIR.endswith('/pattoo-shared/tests/test_pattoo_shared') is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''\
This script is not installed in the "pattoo-shared/tests/test_pattoo_shared" \
directory. Please fix.''')
    sys.exit(2)

# Pattoo imports
from pattoo_shared import phttp
from pattoo_shared.configuration import Config
from tests.libraries.configuration import UnittestConfig


class TestPost(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method / function __init__."""
        pass

    def test_post(self):
        """Testing method / function post."""
        pass

    def test_purge(self):
        """Testing method / function purge."""
        pass


class TestPassiveAgent(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test___init__(self):
        """Testing method / function __init__."""
        pass

    def test_relay(self):
        """Testing method / function relay."""
        pass

    def test_get(self):
        """Testing method / function get."""
        pass


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Test the times module."""

# Standard imports
import unittest
import os
import sys

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
if EXEC_DIR.endswith('/pattoo-shared/tests/test_pattoo_shared') is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''\
This script is not installed in the "pattoo-shared/tests/test_pattoo_shared" \
directory. Please fix.''')
    sys.exit(2)

# Pattoo imports
from pattoo_shared import times
from tests.libraries.configuration import UnittestConfig


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################
    def test_validate_timestamp(self):
        """Testing function validate_timestamp."""
        # Initialize key variables
        result = times.validate_timestamp(300, 300)
        self.assertEqual(result, True)
        result = times.validate_timestamp(400, 300)
        self.assertEqual(result, False)
        result = times.validate_timestamp(500, 300)
        self.assertEqual(result, False)
        result = times.validate_timestamp(500, '300')
        self.assertEqual(result, False)
        result = times.validate_timestamp('500', 300)
        self.assertEqual(result, False)
        result = times.validate_timestamp(500, 0)
        self.assertEqual(result, False)

    def test_normalized_timestamp(self):
        """Testing function normalized_timestamp."""
        # Initialize key variables
        polling_interval = 300

        timestamp = 31
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, (0, polling_interval))

        timestamp = 301
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, (300, polling_interval))

        timestamp = 900
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, (900, polling_interval))

        timestamp = 1000
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, (900, polling_interval))

        # Initialize key variables
        polling_interval = 30

        timestamp = 31
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, (30, polling_interval))

        timestamp = 301
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, (300, polling_interval))

        timestamp = 900
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, (900, polling_interval))

        timestamp = 1000
        result = times.normalized_timestamp(
            polling_interval, timestamp=timestamp)
        self.assertEqual(result, (990, polling_interval))


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Pattoo classes that manage various configurations."""

# Standard imports
import os

# Import project libraries
from pattoo_shared import files
from pattoo_shared import log
from pattoo_shared.constants import (
    PATTOO_API_AGENT_PREFIX, PATTOO_API_AGENT_EXECUTABLE)
from pattoo_shared.variables import PollingTarget


class Config(object):
    """Class gathers all configuration information.

    Only processes the following YAML keys in the configuration file:

        main:
        remote_api:

    """

    def __init__(self):
        """Initialize the class.

        Args:
            None

        Returns:
            None

        """
        # Get the configuration directory
        # Expand linux ~ notation for home directories if provided.
        _config_directory = log.check_environment()
        config_directory = os.path.expanduser(_config_directory)
        self._configuration = files.read_yaml_files(config_directory)

    def api_listen_address(self):
        """Get api_listen_address.

        Args:
            None

        Returns:
            result: result

        """
        # Get result
        key = PATTOO_API_AGENT_EXECUTABLE
        sub_key = 'api_listen_address'
        result = search(key, sub_key, self._configuration, die=False)

        # Default to 0.0.0.0
        if result is None:
            result = '0.0.0.0'
        return result

    def polling_interval(self):
        """Get interval.

        Args:
            None

        Returns:
            result: result

        """
        # Get result
        key = 'main'
        sub_key = 'polling_interval'
        intermediate = search(key, sub_key, self._configuration, die=False)

        # Default to 300
        if intermediate is None:
            result = 300
        else:
            result = int(intermediate)
        return result

    def api_ip_address(self):
        """Get api_ip_address.

        Args:
            None

        Returns:
            result: result

        """
        # Initialize key variables
        key = PATTOO_API_AGENT_EXECUTABLE
        sub_key = 'api_ip_address'

        # Get result
        result = search(key, sub_key, self._configuration, die=False)
        if result is None:
            result = 'localhost'
        return result

    def api_ip_bind_port(self):
        """Get api_ip_bind_port.

        Args:
            None

        Returns:
            result: result

        """
        # Initialize key variables
        key = PATTOO_API_AGENT_EXECUTABLE
        sub_key = 'api_ip_bind_port'

        # Get result
        intermediate = search(key, sub_key, self._configuration, die=False)
        if intermediate is None:
            result = 6000
        else:
            result = int(intermediate)
        return result

    def api_uses_https(self):
        """Get api_uses_https.

        Args:
            None

        Returns:
            result: result

        """
        # Initialize key variables
        key = PATTOO_API_AGENT_EXECUTABLE
        sub_key = 'api_uses_https'

        # Get result
        result = search(key, sub_key, self._configuration, die=False)
        if result is None:
            result = False
        return result

    def api_uri(self):
        """Get api_uri.

        Args:
            None

        Returns:
            result: result

        """
        # Return
        result = '{}/receive'.format(PATTOO_API_AGENT_PREFIX)
        return result

    def api_server_url(self, agent_id):
        """Get pattoo server's remote URL.

        Args:
            agent_id: Agent ID

        Returns:
            result: URL.

        """
        # Construct URL for server
        if self.api_uses_https() is True:
            prefix = 'https://'
        else:
            prefix = 'http://'

        # Return
        result = (
            '{}{}:{}{}/{}'.format(
                prefix, self.api_ip_address(),
                self.api_ip_bind_port(), self.api_uri(), agent_id))
        return result

    def log_directory(self):
        """Get log_directory.

        Args:
            None

        Returns:
            result: result

        """
        # Get result
        sub_key = 'log_directory'
        result = None
        key = 'main'

        # Get new result
        _result = search(key, sub_key, self._configuration)

        # Expand linux ~ notation for home directories if provided.
        result = os.path.expanduser(_result)

        # Check if value exists. We cannot use log2die_safe as it does not
        # require a log directory location to work properly
        if os.path.isdir(result) is False:
            log_message = (
                'log_directory: "{}" '
                'in configuration doesn\'t exist!'.format(result))
            log.log2die_safe(1003, log_message)

        # Return
        return result

    def log_file(self):
        """Get log_file.

        Args:
            None

        Returns:
            result: result

        """
        _log_directory = self.log_directory()
        result = '{}{}pattoo.log'.format(_log_directory, os.sep)
        return result

    def log_file_api(self):
        """Get log_file_api.

        Args:
            None

        Returns:
            result: result

        """
        # Get result
        _log_directory = self.log_directory()
        result = '{}{}pattoo-api.log'.format(_log_directory, os.sep)
        return result

    def log_file_daemon(self):
        """Get log_file_daemon.

        Args:
            None

        Returns:
            result: result

        """
        # Get result
        _log_directory = self.log_directory()
        result = '{}{}pattoo-daemon.log'.format(_log_directory, os.sep)
        return result

    def log_level(self):
        """Get log_level.

        Args:
            None

        Returns:
            result: result

        """
        # Get result
        sub_key = 'log_level'
        key = 'main'
        result = None

        # Return
        intermediate = search(key, sub_key, self._configuration, die=False)
        if intermediate is None:
            result = 'debug'
        else:
            result = '{}'.format(intermediate).lower()
        return result

    def cache_directory(self):
        """Determine the cache_directory.

        Args:
            None

        Returns:
            value: configured cache_directory

        """
        # Initialize key variables
        key = 'main'
        sub_key = 'cache_directory'

        # Get result
        _value = search(key, sub_key, self._configuration)

        # Expand linux ~ notation for home directories if provided.
        value = os.path.expanduser(_value)

        # Create directory if it doesn't exist
        files.mkdir(value)

        # Return
        return value

    def daemon_directory(self):
        """Determine the daemon_directory.

        Args:
            None

        Returns:
            value: configured daemon_directory

        """
        # Initialize key variables
        key = 'main'
        sub_key = 'daemon_directory'

        # Get result
        _value = search(key, sub_key, self._configuration)

        # Expand linux ~ notation for home directories if provided.
        value = os.path.expanduser(_value)

        # Create directory if it doesn't exist
        files.mkdir(value)

        # Return
        return value

    def agent_cache_directory(self, agent_program):
        """Get agent_cache_directory.

        Args:
            agent_program: Name of agent

        Returns:
            result: result

        """
        # Get result
        result = '{}/{}'.format(self.cache_directory(), agent_program)

        # Create directory if it doesn't exist
        files.mkdir(result)

        # Return
        return result

    def _polling_targets(self, _data):
        """Create list of PollingTarget objects.

        Args:
            _data: List of dicts with 'address' and 'multiplier' as keys

        Returns:
            results: List of PollingTarget objects

        """
        # Start conversion
        results = []

        if isinstance(_data, list) is True:
            # Cycle through list
            for item in _data:
                # Default multiplier
                multiplier = 1

                # Reject non dict data
                if isinstance(item, dict) is False:
                    continue

                # Assign address value only if present
                if 'address' in item:
                    address = item['address']
                else:
                    continue

                # Assign replacement multiplier
                if 'multiplier' in item:
                    multiplier = item['multiplier']

                # Populate result
                result = PollingTarget(address=address, multiplier=multiplier)
                results.append(result)

        # Return
        return results


def search(key, sub_key, config_dict, die=True):
    """Get config parameter from YAML.

    Args:
        key: Primary key
        sub_key: Secondary key
        config_dict: Dictionary to explore
        die: Die if true and the result encountered is None

    Returns:
        result: result

    """
    # Get result
    result = None

    # Verify config_dict is indeed a dict.
    # Die safely as log_directory is not defined
    if isinstance(config_dict, dict) is False:
        log.log2die_safe(1021, 'Invalid configuration file. YAML not found')

    # Get new result
    if key in config_dict:
        # Make sure we don't have a None value
        if config_dict[key] is None:
            log_message = (
                '{}: value in configuration is blank. Please fix'.format(key))
            log.log2die_safe(1004, log_message)

        # Get value we need
        if sub_key in config_dict[key]:
            result = config_dict[key][sub_key]

    # Error if not configured
    if result is None and die is True:
        log_message = (
            '{}:{} not defined in configuration'.format(key, sub_key))
        log.log2die_safe(1016, log_message)

    # Return
    return result



#!/usr/bin/env python3
"""Test the files module."""

# Standard imports
import unittest
import os
import tempfile
import sys
import json
from math import pi
from random import randint
import shutil

# PIP imports
import yaml

# Try to create a working PYTHONPATH
EXEC_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.abspath(os.path.join(
    os.path.abspath(os.path.join(EXEC_DIR, os.pardir)), os.pardir))
if EXEC_DIR.endswith('/pattoo-shared/tests/test_pattoo_shared') is True:
    # We need to prepend the path in case PattooShared has been installed
    # elsewhere on the system using PIP. This could corrupt expected results
    sys.path.insert(0, ROOT_DIR)
else:
    print('''\
This script is not installed in the "pattoo-shared/tests/test_pattoo_shared" \
directory. Please fix.''')
    sys.exit(2)

# Pattoo imports
from pattoo_shared import files
from tests.libraries.configuration import UnittestConfig


class TestBasicFunctions(unittest.TestCase):
    """Checks all functions and methods."""

    #########################################################################
    # General object setup
    #########################################################################

    def test_read_yaml_files(self):
        """Testing method / function read_yaml_files."""
        # Initializing key variables
        dict_1 = {
            'key1': 1,
            'key2': 2,
            'key3': 3,
            'key4': 4,
        }

        dict_2 = {
            'key6': 6,
            'key7': 7,
        }
        dict_3 = {}

        # Populate a third dictionary with contents of other dictionaries.
        for key, value in dict_1.items():
            dict_3[key] = value

        for key, value in dict_2.items():
            dict_3[key] = value

        # Create temp file with known data
        directory = tempfile.mkdtemp()
        filenames = {
            '{}/file_1.yaml'.format(directory): dict_1,
            '{}/file_2.yaml'.format(directory): dict_2
        }
        for filename, data_dict in filenames.items():
            with open(filename, 'w') as filehandle:
                yaml.dump(data_dict, filehandle, default_flow_style=False)

        # Get Results
        result = files.read_yaml_files(directory)

        # Clean up
        for key in result.keys():
            self.assertEqual(dict_3[key], result[key])
        filelist = [
            next_file for next_file in os.listdir(
                directory) if next_file.endswith('.yaml')]
        for delete_file in filelist:
            delete_path = '{}/{}'.format(directory, delete_file)
            os.remove(delete_path)
        os.removedirs(directory)

    def test_read_yaml_file(self):
        """Testing function read_yaml_file."""
        # Initializing key variables
        dict_1 = {
            'key1': 1,
            'key2': 2,
            'key3': 3,
            'key4': 4,
        }

        # Create a temporary file without a json extension and test
        tmp = tempfile.NamedTemporaryFile(delete=False)
        with self.assertRaises(SystemExit):
            _ = files.read_yaml_file(tmp.name)
        os.remove(tmp.name)

        # Create temp file with known data
        directory = tempfile.mkdtemp()
        file_data = [
            (('{}/file_1.yaml').format(directory), dict_1)
        ]
        for item in file_data:
            filename = item[0]
            data_dict = item[1]
            with open(filename, 'w') as filehandle:
                yaml.dump(data_dict, filehandle, default_flow_style=False)

            # Get Results
            result = files.read_yaml_file(filename)

            # Test equivalence
            for key in result.keys():
                self.assertEqual(data_dict[key], result[key])

        # Clean up
        filelist = [
            next_file for next_file in os.listdir(
                directory) if next_file.endswith('.yaml')]
        for delete_file in filelist:
            delete_path = ('{}/{}').format(directory, delete_file)
            os.remove(delete_path)
        os.removedirs(directory)

    def test_read_json_files(self):
        """Testing method / function read_json_files."""
        # Initializing key variables
        dict_1 = {
            'key1': 1,
            'key2': 2,
            'key3': 3,
            'key4': 4,
        }

        dict_2 = {
            'key6': 6,
            'key7': 7,
        }

        # Create a temporary file without a json extension and test
        directory = tempfile.mkdtemp()
        with self.assertRaises(SystemExit):
            _ = files.read_json_files(directory)
        os.removedirs(directory)

        # Create a temporary file without a json extension and test
        directory = tempfile.mkdtemp()
        with self.assertRaises(SystemExit):
            _ = files.read_json_files(directory, die=True)
        os.removedirs(directory)

        # Test with die being False. Nothing should happen
        directory = tempfile.mkdtemp()
        _ = files.read_json_files(directory, die=False)
        os.removedirs(directory)

        # Create temp file with known data
        directory = tempfile.mkdtemp()
        filenames = {
            '{}/file_1.json'.format(directory): dict_1,
            '{}/file_2.json'.format(directory): dict_2
        }
        for filename, data_dict in filenames.items():
            with open(filename, 'w') as filehandle:
                json.dump(data_dict, filehandle)

        # Get Results
        result = files.read_json_files(directory)

        # First test, only 2 files
        self.assertEqual(len(result), 2)

        # Clean up
        for filepath, data in result:
            self.assertEqual(filepath in filenames, True)
            for key, value in sorted(data.items()):
                self.assertEqual(filenames[filepath][key], value)
            os.remove(filepath)
        os.removedirs(directory)

    def test_read_json_file(self):
        """Testing function read_json_file."""
        # Initialize key variables
        data = {
            'key1': 1,
            'key2': 2,
            'key3': 3,
            'key4': 4,
        }

        # Create a temporary file without a json extension and test
        tmp = tempfile.NamedTemporaryFile(delete=False)
        with self.assertRaises(SystemExit):
            _ = files.read_json_file(tmp.name)
        os.remove(tmp.name)

        # Create a temporary file without a json extension and test
        tmp = tempfile.NamedTemporaryFile(delete=False)
        with self.assertRaises(SystemExit):
            _ = files.read_json_file(tmp.name, die=True)
        os.remove(tmp.name)

        # Test with die being False. Nothing should happen
        tmp = tempfile.NamedTemporaryFile(delete=False)
        _ = files.read_json_file(tmp.name, die=False)
        os.remove(tmp.name)

        # Create json file and test
        tmp = tempfile.NamedTemporaryFile(delete=False, suffix='.json')
        with open(tmp.name, 'w') as f_handle:
            json.dump(data, f_handle)
        result = files.read_json_file(tmp.name)
        self.assertEqual(len(result), 4)
        for key, value in sorted(result.items()):
            self.assertEqual(data[key], value)
        os.remove(tmp.name)

    def test_mkdir(self):
        """Testing function mkdir."""
        # Test with file, not directory
        tmpfile = tempfile.NamedTemporaryFile(delete=False).name
        open(tmpfile, 'a').close()
        with self.assertRaises(SystemExit):
            files.mkdir(tmpfile)
        os.remove(tmpfile)

        # Create a sub directory of a temp directory.
        directory = '/tmp/test_pattoo-unittest/{}.fake'.format(
            randint(1, 10000) * pi)
        files.mkdir(directory)
        self.assertTrue(os.path.isdir(directory))
        shutil.rmtree(directory)


if __name__ == '__main__':
    # Make sure the environment is OK to run unittests
    UnittestConfig().create()

    # Do the unit test
    unittest.main()



#!/usr/bin/env python3
"""Pattoo .Agent class.

Description:

    This script:
        1) Processes a variety of information from agents
        2) Posts the data using HTTP to a server listed
           in the configuration file

"""
# Standard libraries
import textwrap
import sys
import time
import argparse
import ipaddress
import multiprocessing
import os
from random import random

# PIP3 libraries
from gunicorn.app.base import BaseApplication

# Pattoo libraries
from pattoo_shared import daemon
from pattoo_shared import log
from pattoo_shared import data
from pattoo_shared.configuration import Config


class Agent(object):
    """Agent class for daemons."""

    def __init__(self, parent, child=None):
        """Initialize the class.

        Args:
            parent: Name of parent daemon
            child: Name of child daemon

        Returns:
            None

        """
        # Initialize key variables (Parent)
        self.parent = parent
        self.pidfile_parent = daemon.pid_file(parent)
        self.lockfile_parent = daemon.lock_file(parent)

        # Initialize key variables (Child)
        if bool(child) is None:
            self._pidfile_child = None
        else:
            self._pidfile_child = daemon.pid_file(child)

    def name(self):
        """Return agent name.

        Args:
            None

        Returns:
            value: Name of agent

        """
        # Return
        value = self.parent
        return value

    def query(self):
        """Create placeholder method. Do not delete."""
        # Do nothing
        pass


class AgentDaemon(daemon.Daemon):
    """Class that manages agent deamonization."""

    def __init__(self, agent):
        """Initialize the class.

        Args:
            agent: agent object

        Returns:
            None

        """
        # Initialize variables to be used by daemon
        self.agent = agent

        # Call up the base daemon
        daemon.Daemon.__init__(self, agent)

    def run(self):
        """Start polling.

        Args:
            None

        Returns:
            None

        """
        # Start polling. (Poller decides frequency)
        while True:
            self.agent.query()


class AgentCLI(object):
    """Class that manages the agent CLI.

    Args:
        None

    Returns:
        None

    """

    def __init__(self):
        """Initialize the class.

        Args:
            None

        Returns:
            None

        """
        # Initialize key variables
        self.parser = None

    def process(self, additional_help=None):
        """Return all the CLI options.

        Args:
            None

        Returns:
            args: Namespace() containing all of our CLI arguments as objects
                - filename: Path to the configuration file

        """
        # Header for the help menu of the application
        parser = argparse.ArgumentParser(
            description=additional_help,
            formatter_class=argparse.RawTextHelpFormatter)

        # CLI argument for starting
        parser.add_argument(
            '--start',
            required=False,
            default=False,
            action='store_true',
            help='Start the agent daemon.'
        )

        # CLI argument for stopping
        parser.add_argument(
            '--stop',
            required=False,
            default=False,
            action='store_true',
            help='Stop the agent daemon.'
        )

        # CLI argument for getting the status of the daemon
        parser.add_argument(
            '--status',
            required=False,
            default=False,
            action='store_true',
            help='Get daemon daemon status.'
        )

        # CLI argument for restarting
        parser.add_argument(
            '--restart',
            required=False,
            default=False,
            action='store_true',
            help='Restart the agent daemon.'
        )

        # CLI argument for stopping
        parser.add_argument(
            '--force',
            required=False,
            default=False,
            action='store_true',
            help=textwrap.fill(
                'Stops or restarts the agent daemon ungracefully when '
                'used with --stop or --restart.', width=80)
        )

        # Get the parser value
        self.parser = parser

    def control(self, agent):
        """Control the pattoo agent from the CLI.

        Args:
            agent: Agent object

        Returns:
            None

        """
        # Get the CLI arguments
        self.process()
        parser = self.parser
        args = parser.parse_args()

        # Run daemon
        _daemon = AgentDaemon(agent)
        if args.start is True:
            _daemon.start()
        elif args.stop is True:
            if args.force is True:
                _daemon.force()
            else:
                _daemon.stop()
        elif args.restart is True:
            if args.force is True:
                _daemon.force()
                _daemon.start()
            else:
                _daemon.restart()
        elif args.status is True:
            _daemon.status()
        else:
            parser.print_help()
            sys.exit(2)


class AgentAPI(Agent):
    """pattoo API agent that serves web pages.

    Args:
        None

    Returns:
        None

    """

    def __init__(self, parent, child, agent_api_variable, app):
        """Initialize the class.

        Args:
            parent: Name of parent daemon
            child: Name of child daemon
            agent_api_variable: AgentAPIVariable object
            app: Flask App

        Returns:
            None

        """
        # Initialize key variables
        Agent.__init__(self, parent, child)
        self._app = app
        self._agent_api_variable = agent_api_variable

    def query(self):
        """Query all remote devices for data.

        Args:
            None

        Returns:
            None

        """
        # Initialize key variables
        config = Config()

        # Check for lock and pid files
        if os.path.exists(self.lockfile_parent) is True:
            log_message = ('''\
Lock file {} exists. Multiple API daemons running API may have died \
catastrophically in the past, in which case the lockfile should be deleted.\
'''.format(self.lockfile_parent))
            log.log2see(1083, log_message)

        if os.path.exists(self.pidfile_parent) is True:
            log_message = ('''\
PID file: {} already exists. Daemon already running? If not, it may have died \
catastrophically in the past in which case you should use --stop --force to \
fix.'''.format(self.pidfile_parent))
            log.log2see(1084, log_message)

        ######################################################################
        #
        # Assign options in format that the Gunicorn WSGI will accept
        #
        # NOTE! to get a full set of valid options pprint(self.cfg.settings)
        # in the instantiation of _StandaloneApplication. The option names
        # do not exactly match the CLI options found at
        # http://docs.gunicorn.org/en/stable/settings.html
        #
        ######################################################################
        options = {
            'bind': _ip_binding(self._agent_api_variable),
            'accesslog': config.log_file_api(),
            'errorlog': config.log_file_api(),
            'capture_output': True,
            'pidfile': self._pidfile_child,
            'loglevel': config.log_level(),
            'workers': _number_of_workers(),
            'umask': 0o0007,
        }

        # Log so that user running the script from the CLI knows that something
        # is happening
        log_message = (
            'Pattoo API running on {}:{} and logging to file {}.'
            ''.format(
                self._agent_api_variable.listen_address,
                self._agent_api_variable.ip_bind_port,
                config.log_file_api()))
        log.log2info(1022, log_message)

        # Run
        _StandaloneApplication(self._app, options).run()


class _StandaloneApplication(BaseApplication):
    """Class to integrate the Gunicorn WSGI with the Pattoo Flask application.

    Modified from: http://docs.gunicorn.org/en/latest/custom.html

    """

    def __init__(self, app, options=None):
        """Initialize the class.

        args:
            app: Flask application object of type Flask(__name__)
            options: Gunicorn CLI options

        """
        # Initialize key variables
        self.options = options or {}
        self.application = app
        super(_StandaloneApplication, self).__init__()

    def load_config(self):
        """Load the configuration."""
        # Initialize key variables
        config = dict([(key, value) for key, value in self.options.items()
                       if key in self.cfg.settings and value is not None])

        # Assign configuration parameters
        for key, value in config.items():
            self.cfg.set(key.lower(), value)

    def load(self):
        """Run the Flask application throught the Gunicorn WSGI."""
        return self.application


def _number_of_workers():
    """Get the number of CPU cores on this server."""
    return (multiprocessing.cpu_count() * 2) + 1


def _ip_binding(aav):
    """Create IPv4 / IPv6 binding for Gunicorn.

    Args:
        aav: AgentAPIVariable object

    Returns:
        result: bind

    """
    # Initialize key variables
    ip_address = aav.listen_address
    ip_bind_port = aav.ip_bind_port
    result = None

    # Check IP address type
    try:
        ip_object = ipaddress.ip_address(ip_address)
    except:
        result = '{}:{}'.format(ip_address, ip_bind_port)

    if bool(result) is False:
        # Is this an IPv4 address?
        ipv4 = isinstance(ip_object, ipaddress.IPv4Address)
        if ipv4 is True:
            result = '{}:{}'.format(ip_address, ip_bind_port)
        else:
            result = '[{}]:{}'.format(ip_address, ip_bind_port)

    # Return result
    return result


def get_agent_id(agent_name, agent_hostname):
    """Create a permanent UID for the agent_name.

    Args:
        agent_name: Agent name

    Returns:
        agent_id: UID for agent

    """
    # Initialize key variables
    filename = daemon.agent_id_file(agent_name, agent_hostname)

    # Read environment file with UID if it exists
    if os.path.isfile(filename):
        with open(filename) as f_handle:
            agent_id = f_handle.readline()
    else:
        # Create a UID and save
        agent_id = _generate_agent_id()
        with open(filename, 'w+') as env:
            env.write(str(agent_id))

    # Return
    return agent_id


def _generate_agent_id():
    """Generate a UID.

    Args:
        None

    Returns:
        agent_id: the UID

    """
    # Create a UID and save
    prehash = '{}{}{}{}{}'.format(
        random(), random(), random(), random(), time.time())
    agent_id = data.hashstring(prehash)

    # Return
    return agent_id



#!/usr/bin/env python3
"""Pattoo files library."""

import os
import time
import json
import yaml

# Pattoo libraries
from pattoo_shared import log


def read_yaml_files(config_directory):
    """Read the contents of all yaml files in a directory.

    Args:
        config_directory: Directory with configuration files

    Returns:
        config_dict: Single dict of combined yaml read from all files

    """
    # Initialize key variables
    yaml_found = False
    yaml_from_file = ''
    all_yaml_read = ''

    if os.path.isdir(config_directory) is False:
        log_message = (
            'Configuration directory "{}" '
            'doesn\'t exist!'.format(config_directory))
        log.log2die_safe(1026, log_message)

    # Cycle through list of files in directory
    for filename in os.listdir(config_directory):
        # Examine all the '.yaml' files in directory
        if filename.endswith('.yaml'):
            # YAML files found
            yaml_found = True

            # Read file and add to string
            filepath = '{}{}{}'.format(config_directory, os.sep, filename)
            yaml_from_file = read_yaml_file(filepath, as_string=True, die=True)

            # Append yaml from file to all yaml previously read
            all_yaml_read = '{}\n{}'.format(all_yaml_read, yaml_from_file)

    # Verify YAML files found in directory. We cannot use logging as it
    # requires a logfile location from the configuration directory to work
    # properly
    if yaml_found is False:
        log_message = (
            'No configuration files found in directory "{}" with ".yaml" '
            'extension.'.format(config_directory))
        log.log2die_safe(1015, log_message)

    # Return
    config_dict = yaml.safe_load(all_yaml_read)
    return config_dict


def read_yaml_file(filepath, as_string=False, die=True):
    """Read the contents of a YAML file.

    Args:
        filepath: Path to file to be read
        as_string: Return a string if True
        die: Die if there is an error

    Returns:
        result: Dict of yaml read

    """
    # Initialize key variables
    if as_string is False:
        result = {}
    else:
        result = ''

    # Read file
    if filepath.endswith('.yaml'):
        try:
            with open(filepath, 'r') as file_handle:
                yaml_from_file = file_handle.read()
        except:
            log_message = (
                'Error reading file {}. Check permissions, '
                'existence and file syntax.'
                ''.format(filepath))
            if bool(die) is True:
                log.log2die_safe(1006, log_message)
            else:
                log.log2debug(1014, log_message)
                return {}

        # Get result
        if as_string is False:
            try:
                result = yaml.safe_load(yaml_from_file)
            except:
                log_message = (
                    'Error reading file {}. Check permissions, '
                    'existence and file syntax.'
                    ''.format(filepath))
                if bool(die) is True:
                    log.log2die_safe(1001, log_message)
                else:
                    log.log2debug(1002, log_message)
                    return {}
        else:
            result = yaml_from_file

    else:
        # Die if not a YAML file
        log_message = '{} is not a YAML file.'.format(filepath)
        if bool(die) is True:
            log.log2die_safe(1065, log_message)
        else:
            log.log2debug(1005, log_message)
            if bool(as_string) is False:
                return {}
            else:
                return ''

    # Return
    return result


def read_json_files(_directory, die=True, age=0, count=None):
    """Read the contents of all JSON files in a directory.

    Args:
        _directory: Directory with JSON files
        die: Die if there is an error
        age: Minimum age of files in seconds
        count: Return first X number of sorted filenames is not None

    Returns:
        result: sorted list of tuples containing JSON read from each file and
            filepath. Sorting is important as it causes the files with the
            older timestamp names to be processed first. This allows the
            last_timestamp column to be incrementally processed versus some
            unexpected order. [(filepath, JSON), (filepath, JSON) ...]

    """
    # Initialize key variables
    json_found = False
    result = []
    processed = 0

    if os.path.isdir(_directory) is False:
        log_message = 'Directory "{}" doesn\'t exist!'.format(_directory)
        log.log2die(1009, log_message)

    # Cycle through list of files in directory
    for filename in sorted(os.listdir(_directory)):
        # Examine all the '.json' files in directory
        if filename.endswith('.json'):
            # JSON files found
            json_found = True

            # Read file and add to tuple list
            filepath = '{}{}{}'.format(_directory, os.sep, filename)
            fileage = time.time() - os.stat(filepath).st_mtime
            if fileage > age:
                result.append((filepath, read_json_file(filepath, die=die)))

            # Stop if necessary
            processed += 1
            if bool(count) is True:
                if processed == count:
                    break

    # Verify JSON files found in directory. We cannot use logging as it
    # requires a logfile location from the configuration directory to work
    # properly
    if (json_found is False) and (bool(die) is True):
        log_message = (
            'No JSON files found in directory "{}" with ".json" '
            'extension.'.format(_directory))
        log.log2die_safe(1102, log_message)

    # Return
    result.sort()
    return result


def read_json_file(filepath, die=True):
    """Read the contents of a YAML file.

    Args:
        filepath: Path to file to be read
        die: Die if there is an error

    Returns:
        result: Dict of JSON read

    """
    # Read file
    if filepath.endswith('.json'):
        try:
            with open(filepath, 'r') as file_handle:
                result = json.load(file_handle)
        except:
            log_message = (
                'Error reading file {}. Check permissions, '
                'existence and file syntax.'
                ''.format(filepath))
            if bool(die) is True:
                log.log2die_safe(1012, log_message)
            else:
                log.log2debug(1013, log_message)
                return {}

    else:
        # Die if not a JSON file
        log_message = '{} is not a JSON file.'.format(filepath)
        if bool(die) is True:
            log.log2die_safe(1010, log_message)
        else:
            log.log2debug(1011, log_message)
            return {}

    # Return
    return result


def mkdir(directory):
    """Create a directory if it doesn't already exist.

    Args:
        directory: Directory name

    Returns:
        None

    """
    # Do work
    if os.path.exists(directory) is False:
        try:
            os.makedirs(directory, mode=0o775)
        except:
            log_message = (
                'Cannot create directory {}.'
                ''.format(directory))
            log.log2die(1090, log_message)

    # Fail if not a directory
    if os.path.isdir(directory) is False:
        log_message = (
            '{} is not a directory.'
            ''.format(directory))
        log.log2die(1043, log_message)



#!/usr/bin/env python3
"""Setup pattoo-shared package."""

# Import setup
import setuptools

# Read the description file
with open('README.rst', 'r') as fh:
    LONG_DESCRIPTION = fh.read()

setuptools.setup(
    # Background information on PIP package.
    name='PattooShared',
    url='https://github.com/PalisadoesFoundation/pattoo-shared',
    download_url='https://github.com/PalisadoesFoundation/pattoo-shared',

    # Who to call
    author='Peter Harrison',
    author_email='peter@palisadoes.org',
    maintainer='Peter Harrison',
    maintainer_email='peter@palisadoes.org',

    # The license can be anything you like
    license='MIT',

    # Automatically find the required package dependencies
    packages=setuptools.find_packages(),

    # Create a description of the package
    description='Shared libraries for Pattoo Servers and Agents.',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/x-rst',

    # Minium python version
    python_requires='>=3.5',

    # Set up the version
    version='0.0.32',
)



"""Generic linux daemon base class for python 3.x."""

from __future__ import print_function
import atexit
import signal
import sys
import os
import time

# Pattoo imports
from pattoo_shared import log
from pattoo_shared import files
from pattoo_shared.configuration import Config


class Daemon(object):
    """A generic daemon class.

    Usage: subclass the daemon class and override the run() method.

    Modified from http://www.jejik.com/files/examples/daemon3x.py

    """

    def __init__(self, agent):
        """Initialize the class.

        Args:
            agent: Agent object

        Returns:
            None

        """
        self.name = agent.name()
        self.pidfile = agent.pidfile_parent
        self.lockfile = agent.lockfile_parent
        self._config = Config()

    def _daemonize(self):
        """Deamonize class. UNIX double fork mechanism.

        Args:
            None

        Returns:
            None

        """
        # Initialize key variables
        daemon_log_file = self._config.log_file_daemon()

        # Create a parent process that will manage the child
        # when the code using this class is done.
        try:
            pid = os.fork()
            if pid > 0:
                # Exit first parent
                sys.exit(0)
        except OSError as err:
            log_message = 'Daemon fork #1 failed: {}'.format(err)
            log_message = '{} - PID file: {}'.format(log_message, self.pidfile)
            log.log2die(1060, log_message)

        # Decouple from parent environment
        os.chdir('/')
        os.setsid()
        os.umask(0)

        # Do second fork
        try:
            pid = os.fork()
            if pid > 0:

                # exit from second parent
                sys.exit(0)
        except OSError as err:
            log_message = 'Daemon fork #2 failed: {}'.format(err)
            log_message = '{} - PID file: {}'.format(log_message, self.pidfile)
            log.log2die(1061, log_message)

        # Redirect standard file descriptors
        sys.stdout.flush()
        sys.stderr.flush()
        f_handle_si = open(os.devnull, 'r')
        f_handle_so = open(daemon_log_file, 'a+')
        f_handle_se = open(daemon_log_file, 'a+')
        os.dup2(f_handle_si.fileno(), sys.stdin.fileno())
        os.dup2(f_handle_so.fileno(), sys.stdout.fileno())
        os.dup2(f_handle_se.fileno(), sys.stderr.fileno())

        # write pidfile
        atexit.register(self.delpid)
        pid = str(os.getpid())
        with open(self.pidfile, 'w+') as f_handle:
            f_handle.write('{}\n'.format(pid))

    def delpid(self):
        """Delete the PID file.

        Args:
            None

        Returns:
            None

        """
        # Delete file
        if os.path.exists(self.pidfile) is True:
            os.remove(self.pidfile)

    def dellock(self):
        """Delete the lock file.

        Args:
            None

        Returns:
            None

        """
        # Delete file
        if self.lockfile is not None:
            if os.path.exists(self.lockfile) is True:
                os.remove(self.lockfile)

    def start(self):
        """Start the daemon.

        Args:
            None

        Returns:
            None

        """
        # Check for a pidfile to see if the daemon already runs
        try:
            with open(self.pidfile, 'r') as pf_handle:
                pid = int(pf_handle.read().strip())

        except IOError:
            pid = None

        if pid:
            log_message = (
                'PID file: {} already exists. Daemon already running?'
                ''.format(self.pidfile))
            log.log2die(1062, log_message)

        # Start the daemon
        self._daemonize()

        # Log success
        log_message = (
            'Daemon {} started - PID file: {}'
            ''.format(self.name, self.pidfile))
        log.log2info(1070, log_message)

        # Run code for daemon
        self.run()

    def force(self):
        """Stop the daemon by deleting the lock file first.

        Args:
            None

        Returns:
            None

        """
        # Delete lock file and stop
        self.dellock()
        self.stop()

    def stop(self):
        """Stop the daemon.

        Args:
            None

        Returns:
            None

        """
        # Get the pid from the pidfile
        try:
            with open(self.pidfile, 'r') as pf_handle:
                pid = int(pf_handle.read().strip())
        except IOError:
            pid = None

        if not pid:
            log_message = (
                'PID file: {} does not exist. Daemon not running?'
                ''.format(self.pidfile))
            log.log2warning(1063, log_message)
            # Not an error in a restart
            return

        # Try killing the daemon process
        try:
            while 1:
                if self.lockfile is None:
                    os.kill(pid, signal.SIGTERM)
                else:
                    time.sleep(0.3)
                    if os.path.exists(self.lockfile) is True:
                        continue
                    else:
                        os.kill(pid, signal.SIGTERM)
                time.sleep(0.3)
        except OSError as err:
            error = str(err.args)
            if error.find("No such process") > 0:
                self.delpid()
                self.dellock()
            else:
                log_message = (str(err.args))
                log_message = (
                    '{} - PID file: {}'.format(log_message, self.pidfile))
                log.log2die(1068, log_message)
        except:
            log_message = (
                'Unknown daemon "stop" error for PID file: {}'
                ''.format(self.pidfile))
            log.log2die(1066, log_message)

        # Log success
        self.delpid()
        self.dellock()
        log_message = (
            'Daemon {} stopped - PID file: {}'
            ''.format(self.name, self.pidfile))
        log.log2info(1071, log_message)

    def restart(self):
        """Restart the daemon.

        Args:
            None

        Returns:
            None

        """
        # Restart with a wait period to make sure things shutdown smoothly
        self.stop()
        time.sleep(3)
        self.start()

    def status(self):
        """Get daemon status.

        Args:
            None

        Returns:
            None

        """
        # Get status
        if os.path.exists(self.pidfile) is True:
            print('Daemon is running - {}'.format(self.name))
        else:
            print('Daemon is stopped - {}'.format(self.name))

    def run(self):
        """You should override this method when you subclass Daemon.

        It will be called after the process has been daemonized by
        start() or restart().
        """
        # Simple comment to pass linter
        pass


class _Directory(object):
    """A class for creating the names of hidden directories."""

    def __init__(self):
        """Initialize the class.

        Args:
            None

        Returns:
            None

        """
        # Initialize key variables
        config = Config()
        self._root = config.daemon_directory()

    def pid(self):
        """Define the hidden pid directory.

        Args:
            None

        Returns:
            value: pid directory

        """
        # Return
        value = '{}/pid'.format(self._root)
        return value

    def lock(self):
        """Define the hidden lock directory.

        Args:
            None

        Returns:
            value: lock directory

        """
        # Return
        value = '{}/lock'.format(self._root)
        return value

    def agent_id(self):
        """Define the hidden agent_id directory.

        Args:
            None

        Returns:
            value: agent_id directory

        """
        # Return
        value = '{}/agent_id'.format(self._root)
        return value


class _File(object):
    """A class for creating the names of hidden files."""

    def __init__(self):
        """Initialize the class.

        Args:
            None

        Returns:
            None

        """
        # Initialize key variables
        self._directory = _Directory()

    def pid(self, prefix):
        """Define the hidden pid directory.

        Args:
            prefix: Prefix of file

        Returns:
            value: pid directory

        """
        # Return
        files.mkdir(self._directory.pid())
        value = '{}/{}.pid'.format(self._directory.pid(), prefix)
        return value

    def lock(self, prefix):
        """Define the hidden lock directory.

        Args:
            prefix: Prefix of file

        Returns:
            value: lock directory

        """
        # Return
        files.mkdir(self._directory.lock())
        value = '{}/{}.lock'.format(self._directory.lock(), prefix)
        return value

    def agent_id(self, agent_name, agent_hostname):
        """Define the hidden agent_id directory.

        Args:
            agent_name: Agent name
            agent_hostname: Agent hostname

        Returns:
            value: agent_id directory

        """
        # Return
        files.mkdir(self._directory.agent_id())
        value = '{}/{}.{}.agent_id'.format(
            self._directory.agent_id(), agent_hostname, agent_name)
        return value


def pid_file(agent_name):
    """Get the pidfile for an agent.

    Args:
        agent_name: Agent name

    Returns:
        result: Name of pid file

    """
    # Return
    f_obj = _File()
    result = f_obj.pid(agent_name)
    return result


def lock_file(agent_name):
    """Get the lockfile for an agent.

    Args:
        agent_name: Agent name

    Returns:
        result: Name of lock file

    """
    # Return
    f_obj = _File()
    result = f_obj.lock(agent_name)
    return result


def agent_id_file(agent_name, agent_hostname):
    """Get the agent_idfile for an agent.

    Args:
        agent_name: Agent name
        agent_hostname: Agent hostname

    Returns:
        result: Name of agent_id file

    """
    # Return
    f_obj = _File()
    result = f_obj.agent_id(agent_name, agent_hostname)
    return result
